@@ -1,6 +1,6 @@
 // AV_compat.cpp
 //
-// Copyright 2012 Roan Trail, Inc.
+// Copyright 2012-2013 Roan Trail, Inc.
 //
 // This file is part of Kinetophone.
 //
@@ -19,13 +19,18 @@
 #include "AV_compat.hpp"
 #include "Sound_file_config.hpp"
 #include <string>
+extern "C"
+{
+#include <libavcodec/avcodec.h>
+#include <libavformat/avformat.h>
+}
 
 using std::string;
 using Roan_trail::Recorder::Sound_file_config;
 using namespace Roan_trail::Builder;
 
 // Note:
-// The ffmpeg file: libavformat/isom.c lists the supported codecs for some containers, including "mov"
+// The FFmpeg file: libavformat/isom.c lists the supported codecs for some containers, including "mov"
 
 //
 // Constructor/destructor
@@ -33,7 +38,9 @@ using namespace Roan_trail::Builder;
 
 AV_compat::AV_compat()
 {
-#if LIBAVFORMAT_VERSION_INT < AV_VERSION_INT(53, 24, 2) // (libavformat 53.24.2 in ffmpeg release 0.9.1)
+  // avformat_network_init was introduced in FFmpeg 0.9
+  // so don't do it until then
+#if LIBAVFORMAT_VERSION_INT < AV_VERSION_INT(53, 24, 0) // FFmpeg (0.9)
   //
 #else
   //
@@ -44,7 +51,9 @@ AV_compat::AV_compat()
 
 AV_compat::~AV_compat()
 {
-#if LIBAVFORMAT_VERSION_INT < AV_VERSION_INT(53, 24, 2) // (libavformat 53.24.2 in ffmpeg release 0.9.1)
+  // avformat_network_deinit was introduced in FFmpeg 0.9
+  // so don't do it until then
+#if LIBAVFORMAT_VERSION_INT < AV_VERSION_INT(53, 24, 0) // FFmpeg (0.9)
   //
 #else
   //
@@ -58,23 +67,23 @@ AV_compat::~AV_compat()
 //
 
 bool AV_compat::codec_ID_for_sound_file_config(const Sound_file_config& config, enum CodecID& return_codec_ID)
-#if LIBAVFORMAT_VERSION_INT < AV_VERSION_INT(52, 64, 2) // (libformat 52.64.2 ffmpeg release 0.6.5)
+#if LIBAVFORMAT_VERSION_INT < AV_VERSION_INT(52, 64, 2) // FFmpeg 0.6
 {
   bool return_value = true;
 
   switch (config.data_format)
   {
   case Sound_file_config::data_format_PCMS8:
-    return_codec_ID = CODEC_ID_PCM_S8;
+    return_codec_ID = codec_ID_PCM_S8;
     break;
   case Sound_file_config::data_format_PCM16:
     switch (config.endianness)
     {
     case Sound_file_config::endian_little:
-      return_codec_ID = CODEC_ID_PCM_S16LE;
+      return_codec_ID = codec_ID_PCM_S16LE;
       break;
     case Sound_file_config::endian_big:
-      return_codec_ID = CODEC_ID_PCM_S16BE;
+      return_codec_ID = codec_ID_PCM_S16BE;
       break;
     default:
       return_value = false;
@@ -85,10 +94,10 @@ bool AV_compat::codec_ID_for_sound_file_config(const Sound_file_config& config,
     switch (config.endianness)
     {
     case Sound_file_config::endian_little:
-      return_codec_ID = CODEC_ID_PCM_S24LE;
+      return_codec_ID = codec_ID_PCM_S24LE;
       break;
     case Sound_file_config::endian_big:
-      return_codec_ID = CODEC_ID_PCM_S24BE;
+      return_codec_ID = codec_ID_PCM_S24BE;
       break;
     default:
       return_value = false;
@@ -99,10 +108,10 @@ bool AV_compat::codec_ID_for_sound_file_config(const Sound_file_config& config,
     switch (config.endianness)
     {
     case Sound_file_config::endian_little:
-      return_codec_ID = CODEC_ID_PCM_S32LE;
+      return_codec_ID = codec_ID_PCM_S32LE;
       break;
     case Sound_file_config::endian_big:
-      return_codec_ID = CODEC_ID_PCM_S32BE;
+      return_codec_ID = codec_ID_PCM_S32BE;
       break;
     default:
       return_value = false;
@@ -113,10 +122,10 @@ bool AV_compat::codec_ID_for_sound_file_config(const Sound_file_config& config,
     switch (config.endianness)
     {
     case Sound_file_config::endian_little:
-      return_codec_ID = CODEC_ID_PCM_F32BE; // MOV container only supports F32BE
+      return_codec_ID = codec_ID_PCM_F32BE; // MOV container only supports F32BE (see ./libavformat/isom.c)
       break;
     case Sound_file_config::endian_big:
-      return_codec_ID = CODEC_ID_PCM_F32BE;
+      return_codec_ID = codec_ID_PCM_F32BE;
       break;
     default:
       return_value = false;
@@ -127,10 +136,10 @@ bool AV_compat::codec_ID_for_sound_file_config(const Sound_file_config& config,
     switch (config.endianness)
     {
     case Sound_file_config::endian_little:
-      return_codec_ID = CODEC_ID_PCM_F64BE; // MOV container only supports F64BE
+      return_codec_ID = codec_ID_PCM_F64BE; // MOV container only supports F64BE (see ./libavformat/isom.c)
       break;
     case Sound_file_config::endian_big:
-      return_codec_ID = CODEC_ID_PCM_F64BE;
+      return_codec_ID = codec_ID_PCM_F64BE;
       break;
     default:
       return_value = false;
@@ -153,16 +162,16 @@ bool AV_compat::codec_ID_for_sound_file_config(const Sound_file_config& config,
   switch (config.data_format)
   {
   case Sound_file_config::data_format_PCMS8:
-    return_codec_ID = CODEC_ID_PCM_S8;
+    return_codec_ID = codec_ID_PCM_S8;
     break;
   case Sound_file_config::data_format_PCM16:
     switch (config.endianness)
     {
     case Sound_file_config::endian_little:
-      return_codec_ID = CODEC_ID_PCM_S16LE;
+      return_codec_ID = codec_ID_PCM_S16LE;
       break;
     case Sound_file_config::endian_big:
-      return_codec_ID = CODEC_ID_PCM_S16BE;
+      return_codec_ID = codec_ID_PCM_S16BE;
       break;
     default:
       return_value = false;
@@ -173,10 +182,10 @@ bool AV_compat::codec_ID_for_sound_file_config(const Sound_file_config& config,
     switch (config.endianness)
     {
     case Sound_file_config::endian_little:
-      return_codec_ID = CODEC_ID_PCM_S24LE;
+      return_codec_ID = codec_ID_PCM_S24LE;
       break;
     case Sound_file_config::endian_big:
-      return_codec_ID = CODEC_ID_PCM_S24BE;
+      return_codec_ID = codec_ID_PCM_S24BE;
       break;
     default:
       return_value = false;
@@ -187,10 +196,10 @@ bool AV_compat::codec_ID_for_sound_file_config(const Sound_file_config& config,
     switch (config.endianness)
     {
     case Sound_file_config::endian_little:
-      return_codec_ID = CODEC_ID_PCM_S32LE;
+      return_codec_ID = codec_ID_PCM_S32LE;
       break;
     case Sound_file_config::endian_big:
-      return_codec_ID = CODEC_ID_PCM_S32BE;
+      return_codec_ID = codec_ID_PCM_S32BE;
       break;
     default:
       return_value = false;
@@ -201,10 +210,10 @@ bool AV_compat::codec_ID_for_sound_file_config(const Sound_file_config& config,
     switch (config.endianness)
     {
     case Sound_file_config::endian_little:
-      return_codec_ID = CODEC_ID_PCM_F32LE;
+      return_codec_ID = codec_ID_PCM_F32LE;
       break;
     case Sound_file_config::endian_big:
-      return_codec_ID = CODEC_ID_PCM_F32BE;
+      return_codec_ID = codec_ID_PCM_F32BE;
       break;
     default:
       return_value = false;
@@ -215,10 +224,10 @@ bool AV_compat::codec_ID_for_sound_file_config(const Sound_file_config& config,
     switch (config.endianness)
     {
     case Sound_file_config::endian_little:
-      return_codec_ID = CODEC_ID_PCM_F64LE;
+      return_codec_ID = codec_ID_PCM_F64LE;
       break;
     case Sound_file_config::endian_big:
-      return_codec_ID = CODEC_ID_PCM_F64BE;
+      return_codec_ID = codec_ID_PCM_F64BE;
       break;
     default:
       return_value = false;
@@ -239,7 +248,7 @@ bool AV_compat::codec_ID_for_sound_file_config(const Sound_file_config& config,
 bool AV_compat::endianness_for_output_format(const AVOutputFormat* output_format,
                                              const Sound_file_config& config,
                                              Sound_file_config::Endianness& return_endian)
-#if LIBAVFORMAT_VERSION_INT < AV_VERSION_INT(52, 64, 2) // (libformat 52.64.2 ffmpeg release 0.6.5)
+#if LIBAVFORMAT_VERSION_INT < AV_VERSION_INT(52, 64, 2) // FFmpeg 0.6
 {
   bool return_value = true;
 
@@ -302,7 +311,7 @@ bool AV_compat::endianness_for_output_format(const AVOutputFormat* output_format
 }
 #endif
 
-#if LIBAVCODEC_VERSION_INT < AV_VERSION_INT(52, 123, 0) // (libavcodec 52.123.0 ffmpeg release 0.7.11)
+#if LIBAVCODEC_VERSION_INT <= AV_VERSION_INT(52, 123, 0) // FFmpeg 0.7.13
 //
 bool AV_compat::sample_fmt_for_sound_file_config(const Sound_file_config& config,
                                                  enum SampleFormat& return_sample_format)
@@ -346,11 +355,11 @@ bool AV_compat::sample_fmt_for_sound_file_config(const Sound_file_config& config
   return return_value;
 }
 
-#if LIBAVFORMAT_VERSION_INT < AV_VERSION_INT(53, 24, 2) // (libavformat 53.24.2 in ffmpeg release 0.9.1)
+#if LIBAVFORMAT_VERSION_INT < AV_VERSION_INT(53, 24, 0) // FFmpeg 0.9
 //
 #else
 //
-void AV_compat::av_zero_dts(AVFormatContext* context, AVStream* ref_stream)
+void AV_compat::zero_dts(AVFormatContext* context, AVStream* ref_stream)
 {
   precondition(context
                && ref_stream);
@@ -363,3 +372,82 @@ void AV_compat::av_zero_dts(AVFormatContext* context, AVStream* ref_stream)
 }
 //
 #endif
+
+//
+// Class constants
+//
+
+#if LIBAVCODEC_VERSION_INT <= AV_VERSION_INT(52, 123, 0) // FFmpeg 0.7.13
+//
+const enum CodecType AV_compat::codec_type_video;
+const enum CodecType AV_compat::codec_type_audio;
+//
+#else
+//
+const enum AVMediaType AV_compat::codec_type_video;
+const enum AVMediaType AV_compat::codec_type_audio;
+//
+#endif
+const int AV_compat::pkt_flag_key;
+
+#if LIBAVCODEC_VERSION_INT <= AV_VERSION_INT(52, 123, 0) // FFmpeg 0.7.13
+//
+const enum SampleFormat AV_compat::sample_fmt_U8;
+const enum SampleFormat AV_compat::sample_fmt_S16;
+const enum SampleFormat AV_compat::sample_fmt_S32;
+const enum SampleFormat AV_compat::sample_fmt_FLT;
+const enum SampleFormat AV_compat::sample_fmt_DBL;
+//
+const int AV_compat::picture_type_I;
+const int AV_compat::picture_type_P;
+//
+#else
+//
+const enum AVSampleFormat AV_compat::sample_fmt_U8;
+const enum AVSampleFormat AV_compat::sample_fmt_S16;
+const enum AVSampleFormat AV_compat::sample_fmt_S32;
+const enum AVSampleFormat AV_compat::sample_fmt_FLT;
+const enum AVSampleFormat AV_compat::sample_fmt_DBL;
+//
+const enum AVPictureType AV_compat::picture_type_I;
+const enum AVPictureType AV_compat::picture_type_P;
+//
+#endif
+
+const AV_compat::Open_flags AV_compat::read_only;
+const AV_compat::Open_flags AV_compat::write_only;
+const AV_compat::Open_flags AV_compat::read_write;
+
+#if LIBAVCODEC_VERSION_INT < AV_VERSION_INT(54, 59, 100) // FFmpeg 1.0
+//
+const enum CodecID AV_compat::codec_ID_PCM_S8;
+const enum CodecID AV_compat::codec_ID_PCM_S16LE;
+const enum CodecID AV_compat::codec_ID_PCM_S16BE;
+const enum CodecID AV_compat::codec_ID_PCM_S24LE;
+const enum CodecID AV_compat::codec_ID_PCM_S24BE;
+const enum CodecID AV_compat::codec_ID_PCM_S32LE;
+const enum CodecID AV_compat::codec_ID_PCM_S32BE;
+const enum CodecID AV_compat::codec_ID_PCM_F32LE;
+const enum CodecID AV_compat::codec_ID_PCM_F32BE;
+const enum CodecID AV_compat::codec_ID_PCM_F64LE;
+const enum CodecID AV_compat::codec_ID_PCM_F64BE;
+//
+const enum CodecID AV_compat::codec_ID_QTRLE;
+//
+#else
+//
+const enum AVCodecID AV_compat::codec_ID_PCM_S8;
+const enum AVCodecID AV_compat::codec_ID_PCM_S16LE;
+const enum AVCodecID AV_compat::codec_ID_PCM_S16BE;
+const enum AVCodecID AV_compat::codec_ID_PCM_S24LE;
+const enum AVCodecID AV_compat::codec_ID_PCM_S24BE;
+const enum AVCodecID AV_compat::codec_ID_PCM_S32LE;
+const enum AVCodecID AV_compat::codec_ID_PCM_S32BE;
+const enum AVCodecID AV_compat::codec_ID_PCM_F32LE;
+const enum AVCodecID AV_compat::codec_ID_PCM_F32BE;
+const enum AVCodecID AV_compat::codec_ID_PCM_F64LE;
+const enum AVCodecID AV_compat::codec_ID_PCM_F64BE;
+//
+const enum AVCodecID AV_compat::codec_ID_QTRLE;
+//
+#endif
