@@ -39,7 +39,7 @@ namespace Roan_trail
     {
     public:
       // constructor/destructor
-      Async_audio_file_writer(bool enable_overflow_logging);
+      explicit Async_audio_file_writer(bool enable_overflow_logging);
       virtual ~Async_audio_file_writer();
       // accessors
       Long_int frames_written_count() { return __sync_fetch_and_add(&m_frames_written_count, 0x0); }
