@@ -1,6 +1,6 @@
 // Device_enum.hpp
 //
-// Copyright 2011 Roan Trail, Inc.
+// Copyright 2011-2012 Roan Trail, Inc.
 //
 // This file is part of Kinetophone.
 //
@@ -21,14 +21,9 @@
 
 #include <portaudio.h>
 #include <string>
-#include <vector>
-
-using std::string;
-using std::vector;
 
 namespace Roan_trail
 {
-  class Error;
   class Error_param;
 
   namespace Recorder
@@ -37,7 +32,7 @@ namespace Roan_trail
     {
     public:
       static bool list_devices(PaSampleFormat sample_format,
-                               string& return_devices,
+                               std::string& return_devices,
                                Error_param& return_error);
     private:
       // prevent compiler from generating
