@@ -30,6 +30,7 @@
 #include <string>
 #include <vector>
 #include <algorithm>
+#include <cstddef>
 
 namespace Roan_trail
 {
@@ -72,74 +73,7 @@ namespace Roan_trail
     //
 
     //
-    // Helpers
-    //
-
-    namespace
-    {
-      const char* ih_text_for_field(const char *node_name,
-                                    const xmlpp::Node* start_node,
-                                    const std::string& diagnostic_prefix,
-                                    const std::string& file_path,
-                                    Error_param& return_error)
-      {
-        precondition(node_name
-                     && start_node
-                     && !return_error());
-
-        const char* return_value = 0;
-
-        start_error_block();
-
-        const std::vector<xmlpp::Node*> field_nodes = start_node->find(node_name);
-        on_error(!field_nodes.size(),
-                 new Source_error(error_location(),
-                                  Source_error::load,
-                                  diagnostic_prefix + std::string(", node ")
-                                  + start_node->get_path()
-                                  + std::string(" ")
-                                  + std::string(node_name)
-                                  + std::string(" node not found"),
-                                  file_path + ih_format_XML_file_line(start_node->get_line())));
-        const std::vector<xmlpp::Node*> field_text_nodes = field_nodes[0]->find("text()");
-        on_error(!field_text_nodes.size(),
-                 new Source_error(error_location(),
-                                  Source_error::load,
-                                  diagnostic_prefix + std::string(", node ")
-                                  + field_nodes[0]->get_path()
-                                  + std::string(" text node not found"),
-                                  file_path + ih_format_XML_file_line(field_nodes[0]->get_line())));
-        // the reinterpret cast is used to convert unsigned char to char
-        // unfortunately, there seems to be no obvious way around this
-        const char *field_text_content =
-          reinterpret_cast<const char *>(field_text_nodes[0]->cobj()->content);
-        on_error(!field_text_content,
-                 new Source_error(error_location(),
-                                  Source_error::load,
-                                  diagnostic_prefix + std::string(", node ")
-                                  + field_text_nodes[0]->get_path()
-                                  + std::string(" text content not found"),
-                                  file_path
-                                  + ih_format_XML_file_line(field_text_nodes[0]->get_line())));
-
-        return_value = field_text_content;
-        goto exit_point;
-
-        end_error_block();
-
-        default_error_handler_and_cleanup(return_error,
-                                          return_value,
-                                          0);
-        goto exit_point;
-
-      exit_point:
-        postcondition(return_error.is_valid_at_return(return_value));
-        return return_value;
-      }
-    }
-
-    //
-    // Load/save
+    //   Load/save
     //
 
     template <class Image> bool Dir_slide_collection<Image>::load_from_source(const std::string& path,
@@ -157,7 +91,7 @@ namespace Roan_trail
       const bool source_exists = File_manager::path_exists(source_path);
       on_error(!source_exists, new Source_error(error_location(),
                                                 Source_error::invalid_source,
-                                                std::string("Source directory not found: ") + source_path));
+                                                std::string("Source path not found: ") + source_path));
       const bool is_directory = File_manager::path_is_directory(source_path);
       on_error(!is_directory, new Source_error(error_location(),
                                                Source_error::invalid_source,
@@ -217,15 +151,33 @@ namespace Roan_trail
 
       try
       {
-        const char *directory_text = ih_text_for_field("//directory",
+        const char *directory_text = ih_text_for_field("//source_path",
                                                        XML_node,
                                                        diagnostic_prefix,
                                                        file_path,
                                                        error);
+        // also try "directory" for this field name
+        if (!directory_text)
+        {
+          directory_text = ih_text_for_field("//directory",
+                                             XML_node,
+                                             diagnostic_prefix,
+                                             file_path,
+                                             error);
+        }
         on_error(!directory_text, new Source_error(error_location(),
                                                    Source_error::general,
                                                    error()));
-        std::string directory = (("" == custom_source_path) ? directory_text : custom_source_path);
+        std::string source_path = (("" == custom_source_path) ? directory_text : custom_source_path);
+        const bool source_exists = File_manager::path_exists(source_path);
+        on_error(!source_exists, new Source_error(error_location(),
+                                                  Source_error::invalid_source,
+                                                  std::string("Source path not found: ") + source_path));
+        const bool is_directory = File_manager::path_is_directory(source_path);
+        on_error(!is_directory, new Source_error(error_location(),
+                                                 Source_error::invalid_source,
+                                                 std::string("Source path is not a directory: ")
+                                               + source_path));
         const std::vector<xmlpp::Node*> slide_nodes = XML_node->find("//slide");
         on_error(!slide_nodes.size(), new Source_error(error_location(),
                                                        Source_error::load,
@@ -234,6 +186,8 @@ namespace Roan_trail
                                                        + std::string(" no slides found"),
                                                        file_path
                                                        + ih_format_XML_file_line(XML_node->get_line())));
+
+        std::vector<Slide<Image>*> slides;
         for (std::vector<xmlpp::Node*>::const_iterator current_slide_node = slide_nodes.begin();
              current_slide_node != slide_nodes.end();
              ++current_slide_node)
@@ -247,7 +201,7 @@ namespace Roan_trail
           on_error(!slide_index_text, new Source_error(error_location(),
                                                        Source_error::general,
                                                        error()));
-          int current_slide_index;
+          size_t current_slide_index;
           std::stringstream current_slide_index_stream(slide_index_text);
           current_slide_index_stream >> current_slide_index;
           // file image name
@@ -274,23 +228,24 @@ namespace Roan_trail
           }
           // else if notes_text is empty, there might be no notes text node, so ignore any error and go on
 
-          const std::string file_image_path = directory + std::string("/") + file_image_name;
-          Slide_collection<Image>::m_slides.push_back(new Slide<Image>(current_slide_index,
-                                                                       slide_notes,
-                                                                       file_image_path));
-          if (Slide_collection<Image>::slide_count() != (current_slide_index + 1))
+          const std::string file_image_path = source_path + std::string("/") + file_image_name;
+          slides.push_back(new Slide<Image>(current_slide_index,
+                                            slide_notes,
+                                            file_image_path));
+          if (slides.size() != (current_slide_index + 1))
           {
             std::stringstream diagnostic_stream;
             diagnostic_stream << diagnostic_prefix;
             diagnostic_stream << ", invalid slide index " << current_slide_index;
-            diagnostic_stream << " found at actual index " << Slide_collection<Image>::slide_count() - 1;
+            diagnostic_stream << " found at actual index " << slides.size() - 1;
             on_error(true, new Source_error(error_location(),
                                             Source_error::load,
                                             file_path));
           }
         }
 
-        Slide_collection<Image>::m_source = directory;
+        Slide_collection<Image>::m_slides = slides;
+        Slide_collection<Image>::m_source = source_path;
         Slide_collection<Image>::m_source_loaded = true;
 
         return_value = true;
@@ -337,7 +292,7 @@ namespace Roan_trail
         xmlpp::Node* slides_node = XML_node->add_child("slides");
         xmlpp::Element* source_type_element = slides_node->add_child("source_type");
         source_type_element->add_child_text(source_type());
-        xmlpp::Element* source_directory_element = slides_node->add_child("directory");
+        xmlpp::Element* source_directory_element = slides_node->add_child("source_path");
         source_directory_element->add_child_text(Slide_collection<Image>::source());
         for (size_t slide_index = 0; slide_index < Slide_collection<Image>::m_slides.size(); ++slide_index)
         {
@@ -381,7 +336,7 @@ namespace Roan_trail
     }
 
     //
-    // Protected member functions
+    //   Protected member functions
     //
 
     template <class Image> bool Dir_slide_collection<Image>::mf_invariant(bool check_base_class) const
