@@ -1,6 +1,6 @@
 // Model.hpp
 //
-// Copyright 2011 Roan Trail, Inc.
+// Copyright 2011-2012 Roan Trail, Inc.
 //
 // This file is part of Kinetophone.
 //
@@ -19,15 +19,8 @@
 #ifndef MODEL_HPP_
 #define MODEL_HPP_
 
-#include <vector>
-
-using std::vector;
-
 namespace Roan_trail
 {
-  class Error;
-  class Error_param;
-
   class Model
   {
   public:
