@@ -1,6 +1,6 @@
 // Movie.cpp
 //
-// Copyright 2011-2012 Roan Trail, Inc.
+// Copyright 2011-2013 Roan Trail, Inc.
 //
 // This file is part of Kinetophone.
 //
@@ -31,6 +31,7 @@
 
 using std::string;
 using Roan_trail::Long_int;
+using Roan_trail::Builder::AV_compat;
 using namespace Roan_trail::Builder;
 
 //
@@ -48,7 +49,7 @@ Movie::Movie(const Movie_config& config)
     m_temporary_movie_file_path(),
     m_output_format(0),
     m_format_context(0),
-    m_AV_compat_object(new AV_compat) // take care of any ffmpeg compatibility initialization
+    m_AV_compat_object(new AV_compat) // take care of any FFmpeg compatibility initialization
 {
   if (!m_class_initialized)
   {
@@ -66,7 +67,7 @@ Movie::~Movie()
 
   mf_cleanup();
 
-  delete m_AV_compat_object; // take care of any ffmpe compatibility de-initialization
+  delete m_AV_compat_object; // take care of any FFmpeg compatibility de-initialization
 }
 
 //
@@ -99,12 +100,12 @@ bool Movie::setup(Sound_source* sound_source, Error_param& return_error)
 
   start_error_block();
 
-  // initialization for FFMPEG
+  // initialization for FFmpeg
   //   setup format
   const string extension = Movie_config::extension_for_movie_preset(m_config.movie_preset);
-  output_format = AV_compat::av_guess_format(extension.c_str(),
-                                             0,
-                                             0);
+  output_format = AV_compat::guess_format(extension.c_str(),
+                                          0,
+                                          0);
   on_error(!output_format, new Build_error(error_location(),
                                            Build_error::setup,
                                            "",
@@ -123,7 +124,10 @@ bool Movie::setup(Sound_source* sound_source, Error_param& return_error)
   // add and setup video track
   int track_ID = 0;
   Frame_rate video_frame_rate;
-  Movie_config::frame_rate_for_frame_rate_type(m_config.frame_rate, video_frame_rate);
+  const bool use_high_resolution_video_timescale = AV_compat::need_high_resolution_video_timescale();
+  Movie_config::frame_rate_for_frame_rate_type(m_config.frame_rate,
+                                               use_high_resolution_video_timescale,
+                                               video_frame_rate);
   Rect_size output_frame_size;
   if (!Movie_config::output_size_for_movie_config(m_config, output_frame_size))
   {
@@ -244,9 +248,9 @@ bool Movie::start(const string& movie_path,
                                                     error()));
 
   // open the temporary movie file for output
-  const int opened_output_file = AV_compat::avio_open(&m_format_context->pb,
-                                                      temporary_file_path.c_str(),
-                                                      URL_WRONLY);
+  const int opened_output_file = AV_compat::io_open(&m_format_context->pb,
+                                                    temporary_file_path.c_str(),
+                                                    AV_compat::write_only);
   close(file_descriptor); // don't need the file descriptor any more, ignore error
   on_error(opened_output_file, new Build_error(error_location(),
                                                Build_error::temporary_movie_create,
@@ -256,10 +260,10 @@ bool Movie::start(const string& movie_path,
   have_open_file = true;
 
   // reset dts position (needed for multiple movies)
-  AV_compat::av_zero_dts(m_format_context, m_video_track->stream());
+  AV_compat::zero_dts(m_format_context, m_video_track->stream());
 
   // write stream header, if necessary
-  const int wrote_header = AV_compat::avformat_write_header(m_format_context);
+  const int wrote_header = AV_compat::write_header(m_format_context);
   on_error(wrote_header, new Build_error(error_location(),
                                          Build_error::temporary_movie_write,
                                          temporary_file_path,
@@ -286,7 +290,7 @@ bool Movie::start(const string& movie_path,
   }
   if (have_open_file)
   {
-    AV_compat::avio_close(m_format_context->pb); // ignore error
+    AV_compat::io_close(m_format_context->pb); // ignore error
   }
   if (("" != temporary_file_path) && File_manager::path_exists(temporary_file_path))
   {
@@ -473,7 +477,7 @@ bool Movie::end(Error_param& return_error)
                                                  "could not write movie trailer"));
 
   need_close = false;
-  const int close_return = AV_compat::avio_close(m_format_context->pb);
+  const int close_return = AV_compat::io_close(m_format_context->pb);
   on_error(close_return, new Build_error(error_location(),
                                          Build_error::temporary_movie_close,
                                          m_temporary_movie_file_path,
@@ -504,7 +508,7 @@ bool Movie::end(Error_param& return_error)
   }
   if (need_close)
   {
-    AV_compat::avio_close(m_format_context->pb); // ignore error
+    AV_compat::io_close(m_format_context->pb); // ignore error
   }
   m_started = false;
   return_value = false;
@@ -597,7 +601,7 @@ void Movie::mf_cleanup()
   if (m_started)
   {
     av_write_trailer(m_format_context); // ignore error
-    AV_compat::avio_close(m_format_context->pb);
+    AV_compat::io_close(m_format_context->pb);
     m_started = false;
   }
   if (m_sound_track)
