@@ -43,7 +43,7 @@ namespace Roan_trail
     {
     public:
       // constructor/destructor/initialization
-      Movie(const Movie_config& config);
+      explicit Movie(const Movie_config& config);
       virtual ~Movie();
       // accessors
       const std::string& temporary_movie_file_path() const { return m_temporary_movie_file_path; }
@@ -83,7 +83,7 @@ namespace Roan_trail
       // paths
       std::string m_movie_file_path; // current destination movie
       std::string m_temporary_movie_file_path; // current temporary movie
-      // ffmpeg
+      // FFmpeg
       //   format
       AVOutputFormat* m_output_format;
       AVFormatContext* m_format_context;
