@@ -39,6 +39,7 @@
 #include "Movie_builder.hpp"
 #include "Movie_builder_config.hpp"
 #include "Movie_builder_model.hpp"
+#include "AV_compat.hpp"
 #include "Sound_builder.hpp"
 #include "Movie.hpp"
 #include "Movie_config.hpp"
@@ -53,6 +54,7 @@
 #include <sstream>
 #include <vector>
 #include <cmath>
+#include <cstddef>
 #include <sys/stat.h>
 
 using std::endl;
@@ -61,6 +63,7 @@ using std::setfill;
 using std::string;
 using std::stringstream;
 using std::vector;
+using Roan_trail::Builder::AV_compat;
 using Roan_trail::Recorder::Segment;
 using Roan_trail::Recorder::Sound_file_config;
 using Roan_trail::Long_int;
@@ -152,7 +155,10 @@ bool Movie_builder::setup(Error_param& return_error)
   m_movie_file_name = File_manager::replace_file_extension(m_config.output.movie_name);
 
   Frame_rate video_frame_rate;
-  Movie_config::frame_rate_for_frame_rate_type(m_config.movie->frame_rate, video_frame_rate);
+  const bool use_high_resolution_video_timescale = AV_compat::need_high_resolution_video_timescale();
+  Movie_config::frame_rate_for_frame_rate_type(m_config.movie->frame_rate,
+                                               use_high_resolution_video_timescale,
+                                               video_frame_rate);
   m_video_frames_per_reference_frame = (static_cast<double>(video_frame_rate.time_scale())
                                         / static_cast<double>(video_frame_rate.frame_length()))
     / static_cast<double>(m_model.audio_recording_frame_rate());
@@ -1068,12 +1074,16 @@ bool Movie_builder::mf_check_source_audio_sample_rate(Error_param &return_error)
 
   const Sound_file_config& sound_config = m_sound_builder->config();
   const Long_int audio_sample_rate = sound_config.sample_rate;
+  const bool use_high_resolution_video_timescale = AV_compat::need_high_resolution_video_timescale();
   if (!Movie_config::audio_sample_rate_compatible_with_frame_rate_type(audio_sample_rate,
-                                                                       m_config.movie->frame_rate))
+                                                                       m_config.movie->frame_rate,
+                                                                       use_high_resolution_video_timescale))
   {
     stringstream diagnostic;
     Frame_rate movie_frame_rate;
-    Movie_config::frame_rate_for_frame_rate_type(m_config.movie->frame_rate, movie_frame_rate);
+    Movie_config::frame_rate_for_frame_rate_type(m_config.movie->frame_rate,
+                                                 use_high_resolution_video_timescale,
+                                                 movie_frame_rate);
     diagnostic << "The audio sample rate " << audio_sample_rate ;
     diagnostic << " must be evenly divisible by the time scale " << movie_frame_rate.time_scale();
     diagnostic << " to be compatible with the movie frame rate of ";
@@ -1083,6 +1093,7 @@ bool Movie_builder::mf_check_source_audio_sample_rate(Error_param &return_error)
     diagnostic << ":" << endl;
     vector<Frame_rate_type> compatible_frame_rates;
     Movie_config::compatible_frame_rates_for_audio_sample_rate(audio_sample_rate,
+                                                               use_high_resolution_video_timescale,
                                                                compatible_frame_rates);
     if (compatible_frame_rates.size())
     {
