@@ -17,6 +17,8 @@
 // not, see <http://www.gnu.org/licenses/>.
 
 #include "Movie_builder_config.hpp"
+#include "common.hpp"
+#include "Image_types.hpp"
 #include "Movie_config.hpp"
 #include "File_manager.hpp"
 #include <string>
@@ -31,6 +33,8 @@ namespace
 {
   const string ic_default_movie_name = "movie";
   const double ic_default_full_resolution_scale = 1.0;
+  const int ic_default_movie_gap_length = 30;
+  const uint32_t ic_default_PDF_fill_color = 0xFFFFFFFF; // white
 }
 
 //
@@ -67,11 +71,12 @@ Movie_builder_config::Output_struct::Output_struct()
 Movie_builder_config::Movie_attributes_struct::Movie_attributes_struct()
   : square_pixels(false),
     silence_gaps(false),
-    silence_gap_length(30),
+    silence_gap_length(ic_default_movie_gap_length),
     full_resolution_scale(ic_default_full_resolution_scale),
     preserve_source_transparency(true),
     aspect_fill_color(),
     transparency_color(),
+    PDF_fill_color(Color(ic_default_PDF_fill_color)),
     crop_image_to_frame(false),
     image_interpolation(true)
 {
