@@ -19,9 +19,7 @@
 #ifndef MOVIE_BUILDER_CONFIG_HPP_
 #define MOVIE_BUILDER_CONFIG_HPP_
 
-#include "Movie_types.hpp"
 #include "Image_types.hpp"
-
 #include <string>
 
 // helper struct to store movie builder settings
@@ -62,6 +60,7 @@ namespace Roan_trail
         bool preserve_source_transparency;
         Color aspect_fill_color;
         Color transparency_color;
+        Color PDF_fill_color;
         bool crop_image_to_frame;
         bool image_interpolation;
       } movie_attributes;
