@@ -125,17 +125,30 @@ bool Movie_config::frame_rate_type_for_string(const string& frame_rate_type_stri
                                     m_frame_rate_type_map);
 }
 
-bool Movie_config::audio_sample_rate_compatible_with_frame_rate_type(int audio_sample_rate, Frame_rate_type rate)
+void Movie_config::frame_rate_for_frame_rate_type(Frame_rate_type rate,
+                                                  bool use_high_resolution_timescale,
+                                                  Frame_rate& return_frame_rate)
+{
+  return_frame_rate = (use_high_resolution_timescale ? m_frame_rate_map_high_resolution[rate]
+                       : m_frame_rate_map[rate]);
+}
+
+bool Movie_config::audio_sample_rate_compatible_with_frame_rate_type(int audio_sample_rate,
+                                                                     Frame_rate_type rate,
+                                                                     bool use_high_resolution_video_timescale)
 {
   precondition((audio_sample_rate >= 0));
 
   Frame_rate video_frame_rate;
-  frame_rate_for_frame_rate_type(rate, video_frame_rate);
+  frame_rate_for_frame_rate_type(rate,
+                                 use_high_resolution_video_timescale,
+                                 video_frame_rate);
   // audio frame rate is compatible if it is a multiple of the frame rate time scale
   return ((audio_sample_rate % video_frame_rate.time_scale()) == 0);
 }
 
 void Movie_config::compatible_frame_rates_for_audio_sample_rate(int audio_sample_rate,
+                                                                bool use_high_resolution_video_timescale,
                                                                 vector<Frame_rate_type>& return_frame_rates)
 {
   precondition((audio_sample_rate >= 0)
@@ -143,7 +156,9 @@ void Movie_config::compatible_frame_rates_for_audio_sample_rate(int audio_sample
 
   for (Frame_rate_type rate = frame_rate_5_Hz; rate <= frame_rate_60_Hz; ++rate)
   {
-    if (audio_sample_rate_compatible_with_frame_rate_type(audio_sample_rate, rate))
+    if (audio_sample_rate_compatible_with_frame_rate_type(audio_sample_rate,
+                                                          rate,
+                                                          use_high_resolution_video_timescale))
     {
       return_frame_rates.push_back(rate);
     }
@@ -256,6 +271,7 @@ Movie_config::Constant_map Movie_config::m_frame_rate_type_map;
 Movie_config::Constant_map Movie_config::m_frame_format_map;
 map<Movie_config::Movie_preset, string> Movie_config::m_movie_preset_extension_map;
 map<Movie_config::Frame_rate_type, Frame_rate> Movie_config::m_frame_rate_map;
+map<Movie_config::Frame_rate_type, Frame_rate> Movie_config::m_frame_rate_map_high_resolution;
 map<Movie_config::Frame_format, Rect_size> Movie_config::m_frame_format_input_map;
 map<Movie_config::Frame_format, Rect_size> Movie_config::m_frame_format_output_map;
 
@@ -301,6 +317,17 @@ void Movie_config::mf_setup_maps()
   m_frame_rate_map[frame_rate_30_Hz]           = Frame_rate( 3000,  100); // compatible with 48 KHz audio
   m_frame_rate_map[frame_rate_50_Hz]           = Frame_rate(  600,   12); // compatible with 48 KHz audio
   m_frame_rate_map[frame_rate_60_Hz]           = Frame_rate( 6000,  100); // compatible with 48 KHz audio
+  // frame rate (high resolution)
+  m_frame_rate_map_high_resolution[frame_rate_5_Hz]            = Frame_rate(12000, 2400); // compat. with 48 KHz audio
+  m_frame_rate_map_high_resolution[frame_rate_10_Hz]           = Frame_rate(12000, 1200); // compat. with 48 KHz audio
+  m_frame_rate_map_high_resolution[frame_rate_20_Hz]           = Frame_rate(12000,  600); // compat. with 48 KHz audio
+  m_frame_rate_map_high_resolution[frame_rate_24_Hz]           = Frame_rate(12000,  500); // compat. with 48 KHz audio
+  m_frame_rate_map_high_resolution[frame_rate_25_Hz]           = Frame_rate(12000,  480); // compat. with 48 KHz audio
+  m_frame_rate_map_high_resolution[frame_rate_29_97_Hz_exact]  = Frame_rate(30000, 1001);
+  m_frame_rate_map_high_resolution[frame_rate_29_97_Hz_approx] = Frame_rate(11988,  400); // compat. with 47952Hz audio
+  m_frame_rate_map_high_resolution[frame_rate_30_Hz]           = Frame_rate(12000,  400); // compat. with 48 KHz audio
+  m_frame_rate_map_high_resolution[frame_rate_50_Hz]           = Frame_rate(12000,  240); // compat. with 48 KHz audio
+  m_frame_rate_map_high_resolution[frame_rate_60_Hz]           = Frame_rate(12000,  200); // compat. with 48 KHz audio
   // frame format
   //   input size
   m_frame_format_input_map[frame_format_601_ntsc_4_to_3]               = Rect_size( 720,  547);
