@@ -66,11 +66,14 @@ namespace Roan_trail
       { return m_frame_rate_type_map[rate]; }
       static bool frame_rate_type_for_string(const std::string& frame_rate_type_string,
                                              Frame_rate_type& return_frame_rate);
-      static void frame_rate_for_frame_rate_type(Frame_rate_type rate, Frame_rate& return_frame_rate)
-      { return_frame_rate = m_frame_rate_map[rate]; }
+      static void frame_rate_for_frame_rate_type(Frame_rate_type rate,
+                                                 bool use_high_resolution_timescale,
+                                                 Frame_rate& return_frame_rate);
       static bool audio_sample_rate_compatible_with_frame_rate_type(int audio_sample_rate,
-                                                                    Frame_rate_type rate);
+                                                                    Frame_rate_type rate,
+                                                                    bool use_high_resolution_video_timescale);
       static void compatible_frame_rates_for_audio_sample_rate(int audio_sample_rate,
+                                                               bool use_high_resolution_video_timescale,
                                                                std::vector<Frame_rate_type>& return_frame_rates);
       //   frame format
       static std::string string_for_frame_format(Frame_format format)
@@ -117,6 +120,7 @@ namespace Roan_trail
       static Constant_map m_frame_format_map;
       static std::map<Movie_preset, std::string> m_movie_preset_extension_map;
       static std::map<Frame_rate_type, Frame_rate> m_frame_rate_map;
+      static std::map<Frame_rate_type, Frame_rate> m_frame_rate_map_high_resolution;
       static std::map<Frame_format, Rect_size> m_frame_format_input_map;
       static std::map<Frame_format, Rect_size> m_frame_format_output_map;
 
