@@ -1,6 +1,6 @@
 // Slide.hpp
 //
-// Copyright 2011-2012 Roan Trail, Inc.
+// Copyright 2011-2013 Roan Trail, Inc.
 //
 // This file is part of Kinetophone.
 //
@@ -38,6 +38,7 @@ namespace Roan_trail
       Slide(Long_int index,
             const std::string& notes,
             const std::string& path);
+      virtual ~Slide() {}
       // accessors
       virtual const std::string& notes() const { return m_notes; }
       virtual const std::string& path() const { return m_path; } // can be a file or structure path
@@ -71,10 +72,10 @@ namespace Roan_trail
         m_notes(notes),
         m_path(path)
     {
-      precondition((index >= 0)
-                   && path.size());
+      precondition((index >= 0));
 
-      postcondition((m_notes == notes)
+      postcondition((m_index == index)
+                    && (m_notes == notes)
                     && (m_path == path)
                     && mf_invariant(false));
     }
@@ -89,8 +90,7 @@ namespace Roan_trail
 
       bool return_value = false;
 
-      if ((m_index < 0)
-          || (!m_path.size()))
+      if ((m_index < 0))
       {
         goto exit_point;
       }
