@@ -32,6 +32,7 @@
 #include <string>
 #include <vector>
 #include <sstream>
+#include <cstddef>
 #include "error/Source_error.hpp"
 #include "error/Error.hpp"
 
@@ -50,7 +51,9 @@ namespace Roan_trail
       virtual std::string source_type() const = 0;
       int current_slide_index() const { return m_current_slide_index; }
       virtual int slide_count() const { return static_cast<int>(m_slides.size()); }
-      virtual bool image_for_slide_at(int slide_index, Image& return_image) const = 0;
+      virtual bool image_for_slide_at(int slide_index,
+                                      const Rect_size& suggested_size,
+                                      Image& return_image) const = 0;
       virtual void thumbnail_for_slide_at(int slide_index, Image& return_image) const = 0;
       virtual Rect_size max_bounds() const = 0;
       // operators
@@ -91,6 +94,13 @@ namespace Roan_trail
       Slide_collection& operator=(const Slide_collection& collection);
     };
 
+    // free functions
+
+    inline bool source_type_from_XML(const std::string& file_path,
+                                     const xmlpp::Node* XML_node,
+                                     std::string& return_source_type,
+                                     Error_param& return_error);
+
     //
     // Template implementation
     //
@@ -104,6 +114,66 @@ namespace Roan_trail
 
         return line_stream.str();
       }
+
+      const char* ih_text_for_field(const char *node_name,
+                                    const xmlpp::Node* start_node,
+                                    const std::string& diagnostic_prefix,
+                                    const std::string& file_path,
+                                    Error_param& return_error)
+      {
+        precondition(node_name
+                     && start_node
+                     && !return_error());
+
+        const char* return_value = 0;
+
+        start_error_block();
+
+        const std::vector<xmlpp::Node*> field_nodes = start_node->find(node_name);
+        on_error(!field_nodes.size(),
+                 new Source_error(error_location(),
+                                  Source_error::load,
+                                  diagnostic_prefix + std::string(", node ")
+                                  + start_node->get_path()
+                                  + std::string(" ")
+                                  + std::string(node_name)
+                                  + std::string(" node not found"),
+                                  file_path + ih_format_XML_file_line(start_node->get_line())));
+        const std::vector<xmlpp::Node*> field_text_nodes = field_nodes[0]->find("text()");
+        on_error(!field_text_nodes.size(),
+                 new Source_error(error_location(),
+                                  Source_error::load,
+                                  diagnostic_prefix + std::string(", node ")
+                                  + field_nodes[0]->get_path()
+                                  + std::string(" text node not found"),
+                                  file_path + ih_format_XML_file_line(field_nodes[0]->get_line())));
+        // the reinterpret cast is used to convert unsigned char to char
+        // unfortunately, there seems to be no obvious way around this
+        const char *field_text_content =
+          reinterpret_cast<const char *>(field_text_nodes[0]->cobj()->content);
+        on_error(!field_text_content,
+                 new Source_error(error_location(),
+                                  Source_error::load,
+                                  diagnostic_prefix + std::string(", node ")
+                                  + field_text_nodes[0]->get_path()
+                                  + std::string(" text content not found"),
+                                  file_path
+                                  + ih_format_XML_file_line(field_text_nodes[0]->get_line())));
+
+        return_value = field_text_content;
+        goto exit_point;
+
+        end_error_block();
+
+        default_error_handler_and_cleanup(return_error,
+                                          return_value,
+                                          0);
+        goto exit_point;
+
+      exit_point:
+        postcondition(return_error.is_valid_at_return(return_value));
+        return return_value;
+      }
     }
 
     template <class Image> Slide_collection<Image>::~Slide_collection()
@@ -286,6 +356,62 @@ namespace Roan_trail
     exit_point:
       return return_value;
     }
+
+    //
+    // Free functions
+    //
+
+    inline bool source_type_from_XML(const std::string& file_path,
+                                     const xmlpp::Node* XML_node,
+                                     std::string& return_source_type,
+                                     Error_param& return_error)
+    {
+      precondition(XML_node
+                   &&!return_error());
+
+      bool return_value = false;
+
+      start_error_block();
+
+      Error_param error;
+      const std::string diagnostic_prefix = "error loading XML file";
+
+      try
+      {
+        const char *source_type_text = ih_text_for_field("//source_type",
+                                                         XML_node,
+                                                         diagnostic_prefix,
+                                                         file_path,
+                                                         error);
+        on_error(!source_type_text, new Source_error(error_location(),
+                                                     Source_error::general,
+                                                     error()));
+        return_source_type = std::string(source_type_text);
+        return_value = true;
+      }
+      catch (const xmlpp::exception& e)
+      {
+        const std::string diagnostic = diagnostic_prefix + std::string(": ") + e.what();
+        on_error(true, new Source_error(error_location(),
+                                        Source_error::load,
+                                        diagnostic,
+                                        file_path));
+      }
+
+      goto exit_point;
+
+      end_error_block();
+
+      default_error_handler_and_cleanup(return_error,
+                                        return_value,
+                                        false);
+      goto exit_point;
+
+    exit_point:
+      postcondition(return_error.is_valid_at_return(return_value));
+      return return_value;
+    }
+
   }
 }
 
