@@ -1,6 +1,6 @@
 // Sound_track.cpp
 //
-// Copyright 2011-2012 Roan Trail, Inc.
+// Copyright 2011-2013 Roan Trail, Inc.
 //
 // This file is part of Kinetophone.
 //
@@ -114,8 +114,8 @@ bool Sound_track::setup(AVFormatContext* format_context, Error_param& return_err
     goto exit_point;
   }
 
-  // initialization for FFMPEG
-  stream = AV_compat::avformat_new_stream(format_context, m_track_ID);
+  // initialization for FFmpeg
+  stream = AV_compat::new_stream(format_context, m_track_ID);
   on_error(!stream, new Build_error(error_location(),
                                     Build_error::setup,
                                     "",
@@ -130,8 +130,11 @@ bool Sound_track::setup(AVFormatContext* format_context, Error_param& return_err
   {
     assert(false && "unsupported sound file config when setting codec ID");
   }
+
+  codec_context->channel_layout = AV_compat::default_channel_layout(codec_context->channels);
+
   // Note: we currently don't need the sample format because
-  //   no audio encoding is done through ffmpeg, but this is here for completeness
+  //   no audio encoding is done through FFmpeg, but this is here for completeness
   if (!AV_compat::sample_fmt_for_sound_file_config(m_config, codec_context->sample_fmt))
   {
     assert(false && "unsupported data format when setting sample format");
@@ -150,7 +153,7 @@ bool Sound_track::setup(AVFormatContext* format_context, Error_param& return_err
                                    "",
                                    "could not find audio codec"));
 
-  codec_opened = AV_compat::avcodec_open(codec_context, codec);
+  codec_opened = AV_compat::codec_open(codec_context, codec);
   on_error(codec_opened, new Build_error(error_location(),
                                          Build_error::setup,
                                          "",
