@@ -1,6 +1,6 @@
 // Speech_synthesizer.cpp
 //
-// Copyright 2011-2012 Roan Trail, Inc.
+// Copyright 2011-2013 Roan Trail, Inc.
 //
 // This file is part of Kinetophone.
 //
@@ -32,9 +32,10 @@
 #include <iomanip>
 #include <sstream>
 #include <string>
+#include <cerrno> // Standards Rule 17 exception (check_code_ignore)
 #include <cmath>
 #include <cstring>
-#include <cerrno> // Standards Rule 17 exception (check_code_ignore)
+#include <cstddef>
 #include <strings.h>
 #include <fcntl.h>
 
@@ -94,6 +95,8 @@ namespace
 Speech_synthesizer::Speech_synthesizer()
   : m_frames_written_count(0),
     m_speech_initialized(false),
+    m_output_file(0),
+    m_temporary_file_path(""),
     m_config(new Speech_synthesizer_config),
     m_logger(Logger::default_logger()),
     m_have_recording(false),
@@ -191,6 +194,7 @@ bool Speech_synthesizer::startup(const Speech_synthesizer_config& config, Error_
                                            Record_error::general,
                                            error()));
     file_open = true;
+    static_cast<void>(file_open); // avoid set but unused warning
 
     m_output_file = sound_file;
 
@@ -658,7 +662,7 @@ SNDFILE* Speech_synthesizer::mf_open_sound_file(string& return_output_file_path,
 
   return_created_temp_file = false;
 
-  SNDFILE* return_value = false;
+  SNDFILE* return_value = 0;
 
   // variable declarations for error cleanup
   bool created_temp_file = false;
