@@ -1,6 +1,6 @@
 // Video_track.cpp
 //
-// Copyright 2011-2012 Roan Trail, Inc.
+// Copyright 2011-2013 Roan Trail, Inc.
 //
 // This file is part of Kinetophone.
 //
@@ -21,6 +21,7 @@
 #include "Movie_config.hpp"
 #include "error/Build_error.hpp"
 #include <limits>
+#include <cstddef>
 
 using std::numeric_limits;
 using Roan_trail::Long_int;
@@ -92,7 +93,7 @@ bool Video_track::setup(AVFormatContext* format_context, Error_param& return_err
   }
 
   // initialization for FFMPEG
-  stream = AV_compat::avformat_new_stream(format_context, m_track_ID);
+  stream = AV_compat::new_stream(format_context, m_track_ID);
   on_error(!stream, new Build_error(error_location(),
                                     Build_error::setup,
                                     "",
@@ -122,7 +123,7 @@ bool Video_track::setup(AVFormatContext* format_context, Error_param& return_err
                                    "",
                                    "could not find codec for video track"));
 
-  codec_opened = AV_compat::avcodec_open(codec_context, codec);
+  codec_opened = AV_compat::codec_open(codec_context, codec);
   on_error(codec_opened, new Build_error(error_location(),
                                          Build_error::setup,
                                          "",
@@ -137,17 +138,20 @@ bool Video_track::setup(AVFormatContext* format_context, Error_param& return_err
                                           "",
                                           "out of memory allocating output frame for video track"));
   //   frame buffer
-  const Long_int frame_buffer_size = avpicture_get_size(codec_context->pix_fmt,
-                                                        codec_context->width,
-                                                        codec_context->height);
-  //   output buffer
-  output_buffer_size = max(2 * frame_buffer_size, 2 * static_cast<Long_int>(FF_MIN_BUFFER_SIZE));
-  output_buffer = static_cast<uint8_t*>(av_malloc(output_buffer_size));
-  on_error(!output_buffer, new Build_error(error_location(),
-                                           Build_error::setup,
-                                           "",
-                                           "",
-                                           "out of memory allocating output buffer for video track"));
+  if (AV_compat::need_encoder_output_buffer())
+  {
+    const Long_int frame_buffer_size = avpicture_get_size(codec_context->pix_fmt,
+                                                          codec_context->width,
+                                                          codec_context->height);
+    //   output buffer
+    output_buffer_size = max(2 * frame_buffer_size, 4 * static_cast<Long_int>(FF_MIN_BUFFER_SIZE));
+    output_buffer = static_cast<uint8_t*>(av_malloc(output_buffer_size));
+    on_error(!output_buffer, new Build_error(error_location(),
+                                             Build_error::setup,
+                                             "",
+                                             "",
+                                             "out of memory allocating output buffer for video track"));
+  }
 
   // success, update data members
   mf_cleanup();
@@ -200,7 +204,6 @@ bool Video_track::setup(AVFormatContext* format_context, Error_param& return_err
   postcondition(// success
                 ((return_value
                   && m_codec_context
-                  && m_output_buffer
                   && m_output_frame
                   && m_setup)
                 // failure
@@ -262,17 +265,21 @@ bool Video_track::add(const uint8_t* raw_frame,
 
   for (Long_int i = 0; i < frame_count; ++i)
   {
-    int output_size;
-    mf_encode_raw_frame(raw_frame, output_size);
-    assert(output_size && "frame was unexpectedly buffered");
-
     AVPacket packet;
-    av_init_packet(&packet);
+    bool got_packet;
+    const bool encode_return = mf_encode(raw_frame,
+                                         packet,
+                                         got_packet);
+    on_error(!encode_return, new Build_error(error_location(),
+                                             Build_error::temporary_movie_write,
+                                             "",
+                                             "",
+                                             "error encoding video frames"));
+    assert(got_packet && "error, frame unexpectedly buffered");
+
     packet.pts = m_current_track_time;
     packet.duration = m_frame_rate.frame_length();
     packet.stream_index = m_stream->index;
-    packet.data = m_output_buffer;
-    packet.size = output_size;
 
     // write the compressed frame
     const int write_return = av_interleaved_write_frame(m_format_context, &packet);
@@ -291,13 +298,13 @@ bool Video_track::add(const uint8_t* raw_frame,
       assert(((i % m_keyframe_period) || m_codec_context->coded_frame->key_frame)
              && "error, frame should be a key frame in key frame period");
       assert(((i % m_keyframe_period)
-              || (AV_compat::av_picture_type_I == m_codec_context->coded_frame->pict_type))
+              || (AV_compat::picture_type_I == m_codec_context->coded_frame->pict_type))
              && "error, frame should be an I-frame in key frame period");
       //   current frame is not in the key frame period
       assert((!(i % m_keyframe_period) || !m_codec_context->coded_frame->key_frame)
              && "error, frame should be not be a key frame outside the key frame period");
       assert((!(i % m_keyframe_period)
-              || (AV_compat::av_picture_type_P == m_codec_context->coded_frame->pict_type))
+              || (AV_compat::picture_type_P == m_codec_context->coded_frame->pict_type))
              && "error, frame should be a P-frame outside the key frame period");
       //   for the next frame, force I-Frame/keyframe for the frames is key frame period, otherwise
       //   force P-Frames
@@ -309,15 +316,23 @@ bool Video_track::add(const uint8_t* raw_frame,
       //   first frame should be an I-Frame/key frame
       assert(((0 != i) || m_codec_context->coded_frame->key_frame)
              && "error, first frame should be a key frame");
-      assert(((0 != i) || (AV_compat::av_picture_type_I == m_codec_context->coded_frame->pict_type))
+      assert(((0 != i) || (AV_compat::picture_type_I == m_codec_context->coded_frame->pict_type))
              && "error, first frame should be a I-frame");
       assert(((0 == i) || !m_codec_context->coded_frame->key_frame)
              && "error, remaining frames should not be key frames");
-      assert(((0 == i) || (AV_compat::av_picture_type_P == m_codec_context->coded_frame->pict_type))
+      assert(((0 == i) || (AV_compat::picture_type_P == m_codec_context->coded_frame->pict_type))
              && "error, remaining frames should be P-frames");
       //   from now on, frames added in this call should be an P-Frame/non-key frames
       m_codec_context->gop_size = ic_max_int;
     }
+
+    // if we have allocated m_output_buffer, free the packet, but don't free the data buffer
+    if (m_output_buffer)
+    {
+      packet.data = 0;
+      packet.size = 0;
+    }
+    av_free_packet(&packet);
   }
 
   return_value = true;
@@ -409,9 +424,7 @@ bool Video_track::mf_invariant(bool check_base_class) const
        && (!m_format_context
            || !m_stream
            || !m_codec_context
-           || !m_output_frame
-           || !m_output_buffer
-           || !m_output_buffer_size))
+           || !m_output_frame))
       || (!m_setup
           && (m_format_context
               || m_stream
@@ -422,6 +435,12 @@ bool Video_track::mf_invariant(bool check_base_class) const
     goto exit_point;
   }
 
+  if ((m_output_buffer && !m_output_buffer_size)
+      || (!m_output_buffer && m_output_buffer_size))
+  {
+    goto exit_point;
+  }
+
   if (m_started && !m_setup)
   {
     goto exit_point;
@@ -448,17 +467,37 @@ bool Video_track::mf_compressed_size_of_pixels(const uint8_t* raw_frame,
 {
   precondition(m_setup);
 
-  int output_size;
-  mf_encode_raw_frame(raw_frame, output_size);
-  return_compressed_size = static_cast<Long_int>(output_size);
+  AVPacket packet;
+  bool got_packet;
+  const bool encode_return = mf_encode(raw_frame,
+                                       packet,
+                                       got_packet);
+
+  return_compressed_size = static_cast<Long_int>(packet.size);
+
+  if (encode_return && got_packet)
+  {
+    // if we have allocated m_output_buffer, free the packet, but don't free the data buffer
+    if (m_output_buffer)
+    {
+      packet.data = 0;
+      packet.size = 0;
+    }
+    av_free_packet(&packet);
+  }
 
   return true;
 }
 
-void Video_track::mf_encode_raw_frame(const uint8_t* raw_frame, int& return_output_size)
+// if function returns true and return_got_packet is returned as true, caller needs free packet data
+bool Video_track::mf_encode(const uint8_t* raw_frame,
+                            AVPacket& packet,
+                            bool& return_got_packet)
 {
   precondition(m_setup);
 
+  return_got_packet = false;
+
   if (PIX_FMT_RGB24 == m_codec_context->pix_fmt)
   {
     mf_fill_in_output_frame(raw_frame);
@@ -468,13 +507,17 @@ void Video_track::mf_encode_raw_frame(const uint8_t* raw_frame, int& return_outp
     assert(false && "error, only RGB colorspace supported");
   }
 
-  // encode the image
-  const int output_size = avcodec_encode_video(m_codec_context,
-                                               m_output_buffer,
-                                               m_output_buffer_size,
-                                               m_output_frame);
+  av_init_packet(&packet);
+  packet.data = m_output_buffer;
+  packet.size = m_output_buffer_size;
 
-  return_output_size = static_cast<Long_int>(output_size);
+  // encode the image
+  const bool encode_return = AV_compat::encode_video(m_codec_context,
+                                                     &packet,
+                                                     m_output_frame,
+                                                     return_got_packet);
+
+  return encode_return;
 }
 
 void Video_track::mf_setup_codec_context_from_preset(Movie_preset_type preset,
