@@ -1,6 +1,6 @@
 // Video_track.hpp
 //
-// Copyright 2011-2012 Roan Trail, Inc.
+// Copyright 2011-2013 Roan Trail, Inc.
 //
 // This file is part of Kinetophone.
 //
@@ -27,6 +27,7 @@ struct AVFormatContext;
 struct AVStream;
 struct AVCodecContext;
 struct AVFrame;
+struct AVPacket;
 struct SwsContext;
 
 namespace Roan_trail
@@ -75,7 +76,7 @@ namespace Roan_trail
       Rect_size m_output_frame_size;
       Movie_preset_type m_movie_preset;
       Long_int m_current_track_time;
-      // ffmpeg
+      // FFmpeg
       //   format reference
       AVFormatContext* m_format_context;
       //   stream
@@ -92,7 +93,9 @@ namespace Roan_trail
       //
       bool mf_compressed_size_of_pixels(const uint8_t* raw_frame,
                                         Long_int& return_compressed_size);
-      void mf_encode_raw_frame(const uint8_t* raw_frame, int& return_output_size);
+      bool mf_encode(const uint8_t* raw_frame,
+                     AVPacket& packet,
+                     bool& return_got_packet);
       void mf_setup_codec_context_from_preset(Movie_preset_type preset,
                                               AVCodecContext& return_video_codec_context);
       void mf_cleanup();
