@@ -1,6 +1,6 @@
 // View.hpp
 //
-// Copyright 2011 Roan Trail, Inc.
+// Copyright 2011-2012 Roan Trail, Inc.
 //
 // This file is part of Kinetophone.
 //
@@ -16,24 +16,23 @@
 // a copy of the GNU General Public License along with Kinetophone. If
 // not, see <http://www.gnu.org/licenses/>.
 
+// Abstract base, view class
+// (MVC pattern)
+
 #ifndef VIEW_HPP_
 #define VIEW_HPP_
 
-#include <vector>
-
-using std::vector;
-
 namespace Roan_trail
 {
-  class Error;
   class Error_param;
 
   class View
   {
   public:
-    View() {}
     virtual ~View() {}
     virtual bool update(Error_param& return_error) = 0;
+  protected:
+    View() {}
   private:
     // prevent compiler from generating
     View(const View& v);
