@@ -29,7 +29,7 @@ namespace Roan_trail
   {
   public:
     // constructor/destructor/copy
-    Curses_error(const char* curses_function_name, const Error* base_error = 0);
+    explicit Curses_error(const char* curses_function_name, const Error* base_error = 0);
     virtual Curses_error* clone() const { return new Curses_error(*this); }
     // other
     virtual std::string error_class() const { return "Curses_error"; }
