@@ -54,7 +54,7 @@ Error::~Error()
 
 Error& Error::operator=(const Error& error)
 {
-  precondition(mf_invariant());
+  precondition(mf_invariant(false));
 
   if (this == &error)
   {
@@ -81,7 +81,7 @@ Error& Error::operator=(const Error& error)
   }
 
  exit_point:
-  postcondition(mf_invariant());
+  postcondition(mf_invariant(false));
 
   return *this;
 }
@@ -567,12 +567,12 @@ Error_param& Error_param::operator=(Error* error)
 {
   precondition(!m_error
                && m_need_error
-               && mf_invariant());
+               && mf_invariant(false));
 
   m_error = error;
 
   postcondition(m_error == error
-                && mf_invariant());
+                && mf_invariant(false));
 
   return *this;
 }
