@@ -126,7 +126,7 @@ namespace Roan_trail
     // accessors
     bool need_error() const { return m_need_error; }
     // convenience member functions
-    bool is_valid_at_return(bool function_returns_true)
+    bool is_valid_at_return(bool function_returns_true) const
     { return ((function_returns_true && !m_error)
               || (!function_returns_true && ((!m_error && !m_need_error) || (m_error && m_need_error)))); }
   protected:
