@@ -573,9 +573,18 @@ void Kinetophone_build_manager::mf_log_config(const Kinetophone_movie_builder& b
   }
   stringstream hex_stream;
   hex_stream.setf(ios_base::hex);
-  hex_stream << setw(6) << setfill('0');
-  hex_stream << attributes.aspect_fill_color.scalar_value();
-  m_logger << "    aspect fill color (RRGGBB hex): " << hex_stream.str() << endl;
+  if (attributes.aspect_fill_color.has_alpha)
+  {
+   hex_stream << setw(8) << setfill('0');
+   hex_stream << attributes.aspect_fill_color.scalar_value();
+   m_logger << "    aspect fill color (RRGGBBAA hex): " << hex_stream.str() << endl;
+  }
+  else
+  {
+    hex_stream << setw(6) << setfill('0');
+    hex_stream << attributes.aspect_fill_color.scalar_value();
+    m_logger << "    aspect fill color (RRGGBB hex): " << hex_stream.str() << endl;
+  }
   m_logger << "    crop source image to frame: " << (attributes.crop_image_to_frame ? "yes" : "no") << endl;
   m_logger << "    high quality image interpolation when scaling: ";
   m_logger << (attributes.image_interpolation ? "yes" : "no") << endl;
