@@ -98,7 +98,7 @@ namespace
 Kinetophone_builder_app::Kinetophone_builder_app(int argc, const char** argv)
   : Application(argc, argv),
     m_config(new Kinetophone_builder_config),
-    m_model(new Kinetophone_builder_model(*m_config, "directory")),
+    m_model(new Kinetophone_builder_model(*m_config)),
     m_build_manager(new Kinetophone_build_manager(*m_config, *m_model))
 {
   precondition(argv);
