@@ -1,6 +1,6 @@
 // Kinetophone_builder_config.cpp
 //
-// Copyright 2011-2012 Roan Trail, Inc.
+// Copyright 2011-2013 Roan Trail, Inc.
 //
 // This file is part of Kinetophone.
 //
@@ -17,11 +17,13 @@
 // not, see <http://www.gnu.org/licenses/>.
 
 #include "Kinetophone_builder_config.hpp"
+#include "../base/File_manager.hpp"
+#include "../base/Image_types.hpp"
 #include "../base/Movie_builder_config.hpp"
 #include "../base/Movie_config.hpp"
+#include "../base/option_checks.hpp"
+#include "../base/Option_validators.hpp"
 #include "../base/Sound_file_config.hpp"
-#include "../base/Image_types.hpp"
-#include "../base/File_manager.hpp"
 #include "../base/error/Boost_error.hpp"
 #include "../base/error/Kinetophone_error.hpp"
 #include <boost/program_options.hpp>
@@ -29,27 +31,27 @@
 #include <boost/concept_check.hpp>
 #include <exception>
 #include <fstream>
-#include <iomanip>
-#include <istream>
 #include <sstream>
 #include <string>
 #include <cstdlib>
 
 using std::endl;
 using std::exception;
-using std::hex;
 using std::ifstream;
-using std::istream;
-using std::ios_base;
 using std::logic_error;
-using std::ostream;
-using std::setfill;
-using std::setw;
 using std::string;
 using std::stringstream;
-using std::vector;
 using Roan_trail::Color;
 using Roan_trail::File_manager;
+using Roan_trail::Options::check_conflicting_options;
+using Roan_trail::Options::check_option_dependency;
+using Roan_trail::Options::Color_validator;
+using Roan_trail::Options::Directory_validator;
+using Roan_trail::Options::Fraction_validator;
+using Roan_trail::Options::Positive_double_validator;
+using Roan_trail::Options::Positive_int_validator;
+using Roan_trail::Options::Str_to_int_lookup_validator;
+using Roan_trail::Options::Validator;
 using Roan_trail::Builder::Movie_builder_config;
 using namespace Roan_trail::Kinetophone;
 
@@ -65,228 +67,61 @@ namespace
   using Roan_trail::Builder::Movie_config;
   using Roan_trail::Builder::Movie_builder_config;
 
-  // option validation support
-  template<class T> struct Ih_validator
-  {
-    BOOST_CONCEPT_ASSERT((boost::CopyConstructible<T>));
-    BOOST_CONCEPT_ASSERT((boost::DefaultConstructible<T>));
-    Ih_validator(const T& a) : v(a) {}
-    Ih_validator() : v(T()) {}
-    T v;
-  };
+  //
+  // some custom validation code for this configuration
+  //
 
-  template<class T> void validate(boost::any& v, const vector<string>& xs, Ih_validator<T>*, int)
-  {
-    using namespace PO;
-    check_first_occurrence(v);
-    v = boost::any(Ih_validator<T>(boost::lexical_cast<T>(get_single_string(xs))));
-  }
+  typedef struct Str_to_int_lookup_validator<string,
+                                             Sound_file_config::file_type_for_string> Ih_file_type_validator;
+  typedef struct Str_to_int_lookup_validator<string,
+                                             Sound_file_config::data_format_for_string> Ih_data_format_validator;
+  typedef struct Str_to_int_lookup_validator<string,
+                                             Sound_file_config::endianness_for_string> Ih_endianness_validator;
+  typedef struct Str_to_int_lookup_validator<string,
+                                             Movie_config::movie_preset_for_string> Ih_movie_preset_validator;
+  typedef struct Str_to_int_lookup_validator<string,
+                              Movie_config::frame_rate_type_for_string> Ih_frame_rate_type_validator;
+  typedef struct Str_to_int_lookup_validator<string,
+                                             Movie_config::frame_format_for_string> Ih_frame_format_validator;
 
-  template<class T> ostream& operator<<(ostream& os, const Ih_validator<T>& c)
-  {
-    return os << c.v;
-  }
-
-  ostream& operator<<(ostream& os, const Ih_validator<bool>& c)
-  {
-    return os << (c.v ? "yes" : "no");
-  }
-
-  void validate(boost::any& v,
-                const vector<string>& values,
-                Ih_validator<bool>*, int)
-  {
-    using namespace PO;
-
-    check_first_occurrence(v);
-    string s(get_single_string(values, true));
-
-    for (string::size_type i = 0; i < s.size(); ++i)
-    {
-      s[i] = char(tolower(s[i]));
-    }
-
-    if (s.empty() || (s == "on") || (s == "yes") || (s == "1") || (s == "true"))
-    {
-      v = boost::any(Ih_validator<bool>(true));
-    }
-    else if ((s == "off") || (s == "no") || (s == "0") || (s == "false"))
-    {
-      v = boost::any(Ih_validator<bool>(false));
-    }
-    else
-    {
-      throw validation_error(validation_error::invalid_bool_value, s);
-    }
-  }
-
-  typedef bool (*Ih_lookup_ptr)(const string&, int&);
-
-  template<class T, Ih_lookup_ptr F> struct Ih_lookup_validator
-  {
-    BOOST_CONCEPT_ASSERT((boost::CopyConstructible<T>));
-    Ih_lookup_validator(const T& a) : v(a) {}
-    T v;
-  };
-
-  template<class T, Ih_lookup_ptr F> ostream& operator<<(ostream& os,
-                                                         const Ih_lookup_validator<T, F>& c)
-  {
-    return os << c.v;
-  }
-
-  template<class T, Ih_lookup_ptr F> void validate(boost::any& v,
-                const vector<string>& values,
-                Ih_lookup_validator<T, F>*, int)
-  {
-    using namespace PO;
-
-    validators::check_first_occurrence(v);
-    // error if more than one string supplied
-    const string& s = validators::get_single_string(values);
-
-    int type;
-    if (F(s, type))
-    {
-      v = boost::any(Ih_lookup_validator<T, F>(s));
-    }
-    else
-    {
-      throw validation_error(validation_error::invalid_option_value);
-    }
-  }
-
-  template<class T, int i> struct Ih_cond_validator
-  {
-    BOOST_CONCEPT_ASSERT((boost::CopyConstructible<T>));
-    BOOST_CONCEPT_ASSERT((boost::DefaultConstructible<T>));
-    Ih_cond_validator(const T& a) : v(a) {}
-    Ih_cond_validator() : v(T()) {}
-    bool cond() const;
-    T v;
-  };
-
-  template<class T, int i> ostream& operator<<(ostream& os, const Ih_cond_validator<T, i>& c)
-  {
-   return os << c.v;
-  }
-
-  template<class T, int i> void validate(boost::any& v,
-                                         const vector<string>& values,
-                                         Ih_cond_validator<T, i>*, int)
-  {
-    using namespace PO;
-
-    check_first_occurrence(v);
-    string s(get_single_string(values, true));
-
-    T a = boost::lexical_cast<T>(s);
-    Ih_cond_validator<T, i> cv(a);
-
-    if (cv.cond())
-    {
-      v = boost::any(cv);
-    }
-    else
-    {
-      throw validation_error(validation_error::invalid_option_value);
-    }
-  }
-
-  typedef struct Ih_lookup_validator<string, Sound_file_config::file_type_for_string>
-  Ih_file_type_validator;
-  typedef struct Ih_lookup_validator<string, Sound_file_config::data_format_for_string>
-  Ih_data_format_validator;
-  typedef struct Ih_lookup_validator<string, Sound_file_config::endianness_for_string>
-  Ih_endianness_validator;
-
-  typedef struct Ih_lookup_validator<string, Movie_config::movie_preset_for_string>
-  Ih_movie_preset_validator;
-  typedef struct Ih_lookup_validator<string, Movie_config::frame_rate_type_for_string>
-  Ih_frame_rate_type_validator;
-  typedef struct Ih_lookup_validator<string, Movie_config::frame_format_for_string>
-  Ih_frame_format_validator;
-
-  typedef struct Ih_cond_validator<int, 0> Ih_pos_int_validator;
-  template<> bool Ih_pos_int_validator::cond() const
-  {
-    return v > 0;
-  }
-  typedef struct Ih_cond_validator<double, 0> Ih_pos_double_validator;
-  template<> bool Ih_pos_double_validator::cond() const
-  {
-    return v > 0.0;
-  }
-  typedef struct Ih_cond_validator<double, 1> Ih_fraction_validator;
-  template<> bool Ih_fraction_validator::cond() const
-  {
-    return ((v >= 0.0) && (v <= 1.0));
-  }
-
-  struct Ih_color_validator
-  {
-    Ih_color_validator(const Color& c) : v(c) {}
-    Ih_color_validator() : v() {}
-    Color v;
-  };
-
-  template <typename T> struct Ih_hex_to
-  {
-    BOOST_CONCEPT_ASSERT((boost::DefaultConstructible<T>));
-    T v;
-    operator T() const { return v; }
-    friend istream& operator>>(istream& in, Ih_hex_to& out)
-    {
-      in >> hex >> out.v;
-      return in;
-    }
-  };
-
-  void validate(boost::any& v, const vector<string>& xs, Ih_color_validator*, int)
-  {
-    using namespace PO;
-
-    check_first_occurrence(v);
-    const string& s = validators::get_single_string(xs);
-    uint32_t raw_value = boost::lexical_cast<Ih_hex_to<uint32_t> >(s);
-    raw_value <<= 8;
-    Color c(raw_value);
-    v = boost::any(Ih_color_validator(c));
-  }
-
-  ostream& operator<<(ostream& os, const Ih_color_validator& c)
-  {
-    int32_t scalar_color = c.v.scalar_value();
-    if (!c.v.has_alpha)
-    {
-      scalar_color = scalar_color >> 8;
-    }
-    stringstream hex_stream;
-    hex_stream.setf(ios_base::hex);
-    hex_stream << setw(6) << setfill('0');
-    hex_stream << scalar_color;
-
-    return os << hex_stream.str();
-  }
+  //
+  // the main validator struct for this configuration
+  //
 
   struct Ih_validator_struct
   {
     Ih_validator_struct(const Kinetophone_builder_config& s)
-      : session_file_path(s.session_file_path),
-        image_directory(s.image_directory),
+      : image_source_path(s.image_source_path),
+        image_directory(),
+        session_file_path(s.session_file_path),
         movie_builder_config(*s.movie_builder_config),
-        detailed_error(Ih_validator<bool>(s.detailed_error)),
-        quit_on_prebuild_warnings(Ih_validator<bool>(s.quit_on_prebuild_warnings)),
-        custom_installation_dir(s.custom_installation_dir) {}
+        detailed_error(Validator<bool>(s.detailed_error)),
+        quit_on_prebuild_warnings(Validator<bool>(s.quit_on_prebuild_warnings)),
+        custom_installation_dir(Directory_validator(s.custom_installation_dir)) {}
     void update(Kinetophone_builder_config& return_cfg)
     {
       return_cfg.session_file_path = session_file_path;
-      return_cfg.image_directory = image_directory;
       movie_builder_config.update(*return_cfg.movie_builder_config);
       return_cfg.detailed_error = detailed_error.v;
       return_cfg.quit_on_prebuild_warnings = quit_on_prebuild_warnings.v;
-      return_cfg.custom_installation_dir = custom_installation_dir;
+      return_cfg.custom_installation_dir = custom_installation_dir.v;
       // calculate and assign remaining members
+      if ("" == image_source_path)
+      {
+        if ("" != image_directory.v)
+        {
+          return_cfg.image_source_path = image_directory.v;
+        }
+        else
+        {
+          // left blank to indicate that the source in the session file should be used
+          return_cfg.image_source_path = "";
+        }
+      }
+      else
+      {
+        return_cfg.image_source_path = image_source_path;
+      }
       //   movie preset type from name
       string use_movie_preset = movie_builder_config.movie.movie_preset.v;
       if ("" == use_movie_preset)
@@ -340,17 +175,18 @@ namespace
       return_cfg.movie_builder_config->output.movie_name =
         File_manager::file_name_for_file_path(movie_builder_config.output.movie_name);
     }
+    string image_source_path;
+    Directory_validator image_directory;
     string session_file_path;
-    string image_directory;
     struct Movie_builder_config_validator_struct
     {
       Movie_builder_config_validator_struct(const Movie_builder_config& m)
         : movie(*m.movie),
           output(m.output),
           movie_attributes(m.movie_attributes),
-          quiet(Ih_validator<bool>(m.quiet)),
-          verbose(Ih_validator<bool>(m.verbose)),
-          skip_build_warnings(Ih_validator<bool>(m.skip_build_warnings)) {}
+          quiet(Validator<bool>(m.quiet)),
+          verbose(Validator<bool>(m.verbose)),
+          skip_build_warnings(Validator<bool>(m.skip_build_warnings)) {}
       void update(Movie_builder_config& return_m)
       {
         movie.update(*return_m.movie);
@@ -367,7 +203,7 @@ namespace
             frame_rate(Ih_frame_rate_type_validator(Movie_config::string_for_frame_rate_type(m.frame_rate))),
             frame_format(Ih_frame_format_validator(Movie_config::
                                                    string_for_frame_format(m.frame_format))),
-            keyframe_period(Ih_pos_int_validator(m.keyframe_period)),
+            keyframe_period(Positive_int_validator(m.keyframe_period)),
             sound(*m.sound) {}
         void update(Movie_config& return_m)
         {
@@ -389,14 +225,14 @@ namespace
         Ih_movie_preset_validator movie_preset;
         Ih_frame_rate_type_validator frame_rate;
         Ih_frame_format_validator frame_format;
-        Ih_pos_int_validator keyframe_period;
+        Positive_int_validator keyframe_period;
         // currently, none of these can be specified on the command line
         struct Sound_validator_struct
         {
           Sound_validator_struct(const Sound_file_config& s)
-            : channels(Ih_pos_int_validator(s.channels)),
-              sample_rate(Ih_pos_int_validator(static_cast<int>(s.sample_rate))),
-              VBR_quality(Ih_fraction_validator(s.VBR_quality)),
+            : channels(Positive_int_validator(s.channels)),
+              sample_rate(Positive_int_validator(static_cast<int>(s.sample_rate))),
+              VBR_quality(Fraction_validator(s.VBR_quality)),
               file_name(s.file_name),
               file_type(Ih_file_type_validator(Sound_file_config::string_for_file_type(s.file_type))),
               data_format(Ih_data_format_validator(Sound_file_config::string_for_data_format(s.data_format))),
@@ -420,9 +256,9 @@ namespace
               assert(false && "invalid endianness string");
             }
           }
-          Ih_pos_int_validator channels;
-          Ih_pos_int_validator sample_rate;
-          Ih_fraction_validator VBR_quality;
+          Positive_int_validator channels;
+          Positive_int_validator sample_rate;
+          Fraction_validator VBR_quality;
           string file_name;
           Ih_file_type_validator file_type;
           Ih_data_format_validator data_format;
@@ -435,9 +271,9 @@ namespace
           : movie_name(o.movie_name),
             temporary_directory(o.temporary_directory),
             directory(o.directory),
-            file_overwrite(Ih_validator<bool>(o.file_overwrite)),
-            create_multiple_movies(Ih_validator<bool>(o.create_multiple_movies)),
-            temporary_movies(Ih_validator<bool>(o.temporary_movies)) {}
+            file_overwrite(Validator<bool>(o.file_overwrite)),
+            create_multiple_movies(Validator<bool>(o.create_multiple_movies)),
+            temporary_movies(Validator<bool>(o.temporary_movies)) {}
         void update(Movie_builder_config::Output_struct& return_o)
         {
           return_o.movie_name = movie_name;
@@ -450,19 +286,20 @@ namespace
         string movie_name;
         string temporary_directory;
         string directory;
-        Ih_validator<bool> file_overwrite;
-        Ih_validator<bool> create_multiple_movies;
-        Ih_validator<bool> temporary_movies;
+        Validator<bool> file_overwrite;
+        Validator<bool> create_multiple_movies;
+        Validator<bool> temporary_movies;
       } output;
       struct Movie_attributes_validator_struct
       {
         Movie_attributes_validator_struct(const Movie_builder_config::Movie_attributes_struct& a)
-          : square_pixels(Ih_validator<bool>(a.square_pixels)),
-            silence_gaps(Ih_validator<bool>(a.silence_gaps)),
-            silence_gap_length(Ih_pos_int_validator(a.silence_gap_length)),
-            full_resolution_scale(Ih_pos_double_validator(a.full_resolution_scale)),
-            aspect_fill_color(Ih_color_validator(a.aspect_fill_color)),
-            crop_image_to_frame(Ih_validator<bool>(a.crop_image_to_frame)) {}
+          : square_pixels(Validator<bool>(a.square_pixels)),
+            silence_gaps(Validator<bool>(a.silence_gaps)),
+            silence_gap_length(Positive_int_validator(a.silence_gap_length)),
+            full_resolution_scale(Positive_double_validator(a.full_resolution_scale)),
+            aspect_fill_color(Color_validator(a.aspect_fill_color)),
+            PDF_fill_color(Color_validator(a.PDF_fill_color)),
+            crop_image_to_frame(Validator<bool>(a.crop_image_to_frame)) {}
         void update(Movie_builder_config::Movie_attributes_struct& return_a)
         {
           return_a.square_pixels = square_pixels.v;
@@ -470,109 +307,65 @@ namespace
           return_a.silence_gap_length = silence_gap_length.v;
           return_a.full_resolution_scale = full_resolution_scale.v;
           return_a.aspect_fill_color = aspect_fill_color.v;
+          return_a.PDF_fill_color = PDF_fill_color.v;
           return_a.crop_image_to_frame = crop_image_to_frame.v;
         }
-        Ih_validator<bool> square_pixels;
-        Ih_validator<bool> silence_gaps;
-        Ih_pos_int_validator silence_gap_length;
-        Ih_pos_double_validator full_resolution_scale;
-        Ih_color_validator aspect_fill_color;
-        Ih_validator<bool> crop_image_to_frame;
+        Validator<bool> square_pixels;
+        Validator<bool> silence_gaps;
+        Positive_int_validator silence_gap_length;
+        Positive_double_validator full_resolution_scale;
+        Color_validator aspect_fill_color;
+        Color_validator PDF_fill_color;
+        Validator<bool> crop_image_to_frame;
       } movie_attributes;
-      Ih_validator<bool> quiet;
-      Ih_validator<bool> verbose;
-      Ih_validator<bool> skip_build_warnings;
+      Validator<bool> quiet;
+      Validator<bool> verbose;
+      Validator<bool> skip_build_warnings;
     } movie_builder_config;
-    Ih_validator<bool> detailed_error;
-    Ih_validator<bool> quit_on_prebuild_warnings;
-    string custom_installation_dir;
+    Validator<bool> detailed_error;
+    Validator<bool> quit_on_prebuild_warnings;
+    Directory_validator custom_installation_dir;
   };
 
   //
   // Auxiliary functions for checking input validity.
   //
 
-  // Function used to check that 'opt1' and 'opt2' are not specified
-  // at the same time.
-  void ih_check_conflicting_options(const PO::variables_map& VM,
-                                    const char* opt1,
-                                    const char* opt2,
-                                    bool condition = true,
-                                    const string& message = "")
-  {
-    precondition(opt1
-                 && opt2);
-
-    if (condition &&
-        (VM.count(opt1) && !VM[opt1].defaulted()
-         && VM.count(opt2) && !VM[opt2].defaulted()))
-    {
-      string exception_message = string("conflicting options '")
-        + opt1 + "' and '" + opt2 + "'";
-      if ("" != message)
-      {
-        exception_message += string(": ") + message;
-      }
-      exception_message += ".";
-      throw logic_error(exception_message);
-    }
-  }
-
-  // function used to check that 'for_what' is specified, then
-  // 'required_option' is specified too.
-  void ih_check_option_dependency(const PO::variables_map& VM,
-                                  const char* for_what,
-                                  const char* required_option,
-                                  bool condition = true)
-  {
-    precondition(for_what
-                 && required_option);
-
-    if (VM.count(for_what) && !VM[for_what].defaulted())
-    {
-      if ((VM.count(required_option) == 0)
-          || VM[required_option].defaulted()
-          || !condition)
-      {
-        throw logic_error(string("option '") + for_what
-                          + "' requires option '" + required_option + "'.");
-      }
-    }
-  }
-
   void ih_check_option_consistency(const Kinetophone_builder_config& cfg,
                                    const PO::variables_map& VM)
   {
     // conflicting option checks
-    ih_check_conflicting_options(VM,
-                                 "output-directory",
-                                 "movie-name",
-                                 ("" != File_manager::parent_directory_for_path
-                                  (cfg.movie_builder_config->output.movie_name)),
-                                 "output directory specified conflicts with "
-                                 "parent directory of movie name, omit one or the other");
-    ih_check_conflicting_options(VM,
-                                 "quiet",
-                                 "verbose");
-    ih_check_conflicting_options(VM,
-                                 "movie-name",
-                                 "temporary-movies");
-    ih_check_conflicting_options(VM,
-                                 "output-directory",
-                                 "temporary-movies");
-    ih_check_conflicting_options(VM,
-                                 "skip-build-warnings",
-                                 "quit-on-prebuild-warnings");
+    check_conflicting_options(VM,
+                              "output-directory",
+                              "movie-name",
+                              ("" != File_manager::parent_directory_for_path
+                               (cfg.movie_builder_config->output.movie_name)),
+                              "output directory specified conflicts with "
+                              "parent directory of movie name, omit one or the other");
+    check_conflicting_options(VM,
+                              "quiet",
+                              "verbose");
+    check_conflicting_options(VM,
+                              "movie-name",
+                              "temporary-movies");
+    check_conflicting_options(VM,
+                              "output-directory",
+                              "temporary-movies");
+    check_conflicting_options(VM,
+                              "skip-build-warnings",
+                              "quit-on-prebuild-warnings");
+    check_conflicting_options(VM,
+                              "image-directory",
+                              "image-source-path");
     // required option checks
     if ("" == cfg.session_file_path)
     {
       throw logic_error(string("session file not specified"));
     }
     // option dependency checks
-    ih_check_option_dependency(VM,
-                               "silence-gap-length",
-                               "silence-gaps");
-
+    check_option_dependency(VM,
+                            "silence-gap-length",
+                            "silence-gaps");
     // invalid movie preset
     if (Movie_config::movie_preset_unknown == cfg.movie_builder_config->movie->movie_preset)
     {
@@ -607,9 +400,11 @@ namespace
       ("session-file,x", PO::value<string>(&return_v.session_file_path)
        ->default_value(return_v.session_file_path),
        "set narrator session XML file for building movie(s)")
-      ("image-directory,D", PO::value<string>(&return_v.image_directory)
-       ->default_value(return_v.image_directory),
-       "set image source directory (overrides session)")
+      ("image-directory,D", PO::value<Directory_validator>(&return_v.image_directory),
+       "set image source directory (overrides session file)")
+      ("image-source-path,P", PO::value<string>(&return_v.image_source_path)
+       ->default_value(return_v.image_source_path),
+       "set image source path (directory or PDF file, overrides session file)")
       ("sound-file,f", PO::value<string>(&return_v.movie_builder_config.movie.sound.file_name)
        ->default_value(return_v.movie_builder_config.movie.sound.file_name),
        "set sound file path for movie (overrides session)")
@@ -632,66 +427,69 @@ namespace
       ("output-directory,d", PO::value<string>(&return_v.movie_builder_config.output.directory)
        ->default_value(return_v.movie_builder_config.output.directory),
        "set output directory for movie(s)")
-      ("overwrite,o", PO::value<Ih_validator<bool> >(&return_v.movie_builder_config.output.file_overwrite)
-       ->implicit_value(Ih_validator<bool>(true)),
+      ("overwrite,o", PO::value<Validator<bool> >(&return_v.movie_builder_config.output.file_overwrite)
+       ->implicit_value(Validator<bool>(true)),
        "overwrite existing movies")
       ("multiple-movies,u",
-       PO::value<Ih_validator<bool> >(&return_v.movie_builder_config.output.create_multiple_movies)
-       ->implicit_value(Ih_validator<bool>(true)),
+       PO::value<Validator<bool> >(&return_v.movie_builder_config.output.create_multiple_movies)
+       ->implicit_value(Validator<bool>(true)),
        "output each segment as a separate movie")
       ("temporary-movies,T",
-       PO::value<Ih_validator<bool> >(&return_v.movie_builder_config.output.temporary_movies)
-       ->implicit_value(Ih_validator<bool>(true)),
+       PO::value<Validator<bool> >(&return_v.movie_builder_config.output.temporary_movies)
+       ->implicit_value(Validator<bool>(true)),
        "keep temporary movies in the temporary directory, do not move to destination")
       ;
 
     return_desc_params.advanced_desc.add_options()
       ("keyframe-period,K",
-       PO::value<Ih_pos_int_validator>(&return_v.movie_builder_config.movie.keyframe_period),
+       PO::value<Positive_int_validator>(&return_v.movie_builder_config.movie.keyframe_period),
        "set period in frames between video keyframes (must be > 0, can make HUGE movies!)")
       ("square-pixels,q",
-       PO::value<Ih_validator<bool> >(&return_v.movie_builder_config.movie_attributes.square_pixels)
-       ->implicit_value(Ih_validator<bool>(true)),
+       PO::value<Validator<bool> >(&return_v.movie_builder_config.movie_attributes.square_pixels)
+       ->implicit_value(Validator<bool>(true)),
        "force square (1:1 aspect) pixels")
       ("silence-gaps,g",
-       PO::value<Ih_validator<bool> >(&return_v.movie_builder_config.movie_attributes.silence_gaps)
-       ->implicit_value(Ih_validator<bool>(true)),
+       PO::value<Validator<bool> >(&return_v.movie_builder_config.movie_attributes.silence_gaps)
+       ->implicit_value(Validator<bool>(true)),
        "create silence gaps at segment boundaries")
       ("silence-gap-length,l",
-       PO::value<Ih_pos_int_validator>(&return_v.movie_builder_config.movie_attributes.silence_gap_length)
+       PO::value<Positive_int_validator>(&return_v.movie_builder_config.movie_attributes.silence_gap_length)
        ->default_value(return_v.movie_builder_config.movie_attributes.silence_gap_length),
        "set length of silence in video frames (must be > 0, requires --silence-gaps option)")
       ("full-resolution-scale,S",
-       PO::value<Ih_pos_double_validator>(&return_v.movie_builder_config.movie_attributes.full_resolution_scale)
+       PO::value<Positive_double_validator>(&return_v.movie_builder_config.movie_attributes.full_resolution_scale)
        ->default_value(return_v.movie_builder_config.movie_attributes.full_resolution_scale),
        "set scale for output of movies at full resolution (0.0 < scale <= 1.0)")
       ("aspect-fill-color,c",
-       PO::value<Ih_color_validator>(&return_v.movie_builder_config.movie_attributes.aspect_fill_color)
+       PO::value<Color_validator>(&return_v.movie_builder_config.movie_attributes.aspect_fill_color)
        ->default_value(return_v.movie_builder_config.movie_attributes.aspect_fill_color),
        "set color for letterboxing/pillarboxing (format RRGGBB, in hexadecimal")
+      ("PDF-fill-color,L",
+       PO::value<Color_validator>(&return_v.movie_builder_config.movie_attributes.PDF_fill_color)
+       ->default_value(return_v.movie_builder_config.movie_attributes.PDF_fill_color),
+       "set fill color for PDF background (format RRGGBB, in hexadecimal")
       ("crop-image-to-frame,R",
-       PO::value<Ih_validator<bool> >(&return_v.movie_builder_config.movie_attributes.crop_image_to_frame)
-       ->implicit_value(Ih_validator<bool>(true)),
+       PO::value<Validator<bool> >(&return_v.movie_builder_config.movie_attributes.crop_image_to_frame)
+       ->implicit_value(Validator<bool>(true)),
        "scale and fill, rather than fit, source image in video frame ")
       ("skip-build-warnings,k",
-       PO::value<Ih_validator<bool> >(&return_v.movie_builder_config.skip_build_warnings)
-       ->implicit_value(Ih_validator<bool>(true)),
+       PO::value<Validator<bool> >(&return_v.movie_builder_config.skip_build_warnings)
+       ->implicit_value(Validator<bool>(true)),
        "do not output builder warnings (cannot use with --quit-on-prebuild-warnings)")
-      ("quiet,Q", PO::value<Ih_validator<bool> >(&return_v.movie_builder_config.quiet)
-       ->implicit_value(Ih_validator<bool>(true)),
+      ("quiet,Q", PO::value<Validator<bool> >(&return_v.movie_builder_config.quiet)
+       ->implicit_value(Validator<bool>(true)),
        "little output, if any, from the build (cannot use with --verbose)")
-      ("verbose,V", PO::value<Ih_validator<bool> >(&return_v.movie_builder_config.verbose)
-       ->implicit_value(Ih_validator<bool>(true)),
+      ("verbose,V", PO::value<Validator<bool> >(&return_v.movie_builder_config.verbose)
+       ->implicit_value(Validator<bool>(true)),
        "talkative build (cannot use with --quiet)")
-      ("detailed-error,e", PO::value<Ih_validator<bool> >(&return_v.detailed_error)
-       ->implicit_value(Ih_validator<bool>(true)),
+      ("detailed-error,e", PO::value<Validator<bool> >(&return_v.detailed_error)
+       ->implicit_value(Validator<bool>(true)),
        "display detailed diagnostics on error")
-      ("quit-on-prebuild-warnings,p", PO::value<Ih_validator<bool> >(&return_v.quit_on_prebuild_warnings)
-       ->implicit_value(Ih_validator<bool>(true)),
+      ("quit-on-prebuild-warnings,p", PO::value<Validator<bool> >(&return_v.quit_on_prebuild_warnings)
+       ->implicit_value(Validator<bool>(true)),
        "quit, do not continue building, if there are prebuild warning(s) " // (check_code_ignore)
        "(cannot use with --skip-build-warnings)")
-      ("config-directory,C", PO::value<string>(&return_v.custom_installation_dir)
-       ->default_value(return_v.custom_installation_dir),
+      ("config-directory,C", PO::value<Directory_validator>(&return_v.custom_installation_dir),
        "set custom program configuration directory")
       ;
   }
@@ -705,7 +503,7 @@ Kinetophone_builder_config::Kinetophone_builder_config()
   // setup some reasonable defaults
   : help_message(""),
     session_file_path(),
-    image_directory(""), // left blank to indicate that the directory in the session file should be used
+    image_source_path(""), // left blank to indicate that the source in the session file should be used
     movie_builder_config(new Movie_builder_config),
     detailed_error(false),
     quit_on_prebuild_warnings(false),
@@ -771,7 +569,7 @@ int Kinetophone_builder_config::parse_program_options(int argc,
 
     stringstream help_stream;
     help_stream << "Usage:  ";
-    help_stream << "kinetophone_builder --help | [ session_file ] [ options ]" << endl;
+    help_stream << "kinetophone_builder --help | [ session-file ] [ options ]" << endl;
     help_stream << endl;
 
     PO::options_description all_desc("Options");
