@@ -46,7 +46,7 @@ namespace Roan_trail
       // structure members
       std::string help_message;
       std::string session_file_path;
-      std::string image_directory;
+      std::string image_source_path;
       Roan_trail::Builder::Movie_builder_config* movie_builder_config;
       bool detailed_error;
       bool quit_on_prebuild_warnings;
