@@ -1,6 +1,6 @@
 // Kinetophone_builder_model.cpp
 //
-// Copyright 2012 Roan Trail, Inc.
+// Copyright 2012-2013 Roan Trail, Inc.
 //
 // This file is part of Kinetophone.
 //
@@ -23,6 +23,7 @@
 
 #include "Kinetophone_builder_config.hpp"
 #include "../narrator/Kinetophone_dir_slide_collection.hpp"
+#include "../narrator/Kinetophone_PDF_slide_collection.hpp"
 #include "../base/Segment.hpp"
 #include "../base/Slide_collection.hpp"
 #include "../base/error/Kinetophone_error.hpp"
@@ -43,17 +44,17 @@ using Roan_trail::Long_int;
 using Roan_trail::Error_param;
 using Roan_trail::Recorder::Segment;
 using Roan_trail::Source::Slide_collection;
+using Roan_trail::Source::source_type_from_XML;
 using namespace Roan_trail::Kinetophone;
 
 //
 // Constructor/destructor
 //
 
-Kinetophone_builder_model::Kinetophone_builder_model(const Kinetophone_builder_config& builder_config,
-                                                     const string& slides_source_type)
+Kinetophone_builder_model::Kinetophone_builder_model(const Kinetophone_builder_config& builder_config)
   : m_builder_config(builder_config),
     m_slides(0),
-    m_slides_source_type(slides_source_type)
+    m_slides_source(directory_source)
 {
   postcondition(mf_invariant(false));
 }
@@ -90,8 +91,9 @@ bool Kinetophone_builder_model::load_from_XML(const string& file_path, Error_par
   precondition(!return_error()
                && mf_invariant());
 
-  bool return_value = false;
   Slide_collection<RefPtr<Pixbuf> >* slides = 0;
+  Slides_source_type slides_source;
+  bool return_value = false;
 
   start_error_block();
 
@@ -114,17 +116,34 @@ bool Kinetophone_builder_model::load_from_XML(const string& file_path, Error_par
                                                diagnostic_prefix + string(", could not parse"),
                                                file_path));
 
-    // import into the slide collection
-    string custom_source_path;
-    if ("directory" == m_slides_source_type)
+    string source_type_string;
+    const bool got_source_type = source_type_from_XML(file_path,
+                                                      root_node,
+                                                      source_type_string,
+                                                      error);
+    on_error(!got_source_type, new Kinetophone_error(error_location(),
+                                                     Kinetophone_error::file_IO,
+                                                     error()));
+    if ("directory" == source_type_string)
     {
-      custom_source_path = m_builder_config.image_directory;
       slides = new Kinetophone_dir_slide_collection;
+      slides_source = directory_source;
+    }
+    else if ("PDF" == source_type_string)
+    {
+      slides = new Kinetophone_PDF_slide_collection;
+      slides_source = PDF_source;
     }
     else
     {
-      assert(false && "invalid slides source type when creating Kinetophone narrator model");
+      on_error(true, new Kinetophone_error(error_location(),
+                                           Kinetophone_error::file_IO,
+                                           diagnostic_prefix + string(", could not determine source type"),
+                                           file_path));
     }
+
+    // import into the slide collection
+    const string custom_source_path = m_builder_config.image_source_path;
     const bool slides_loaded = slides->load_from_XML(file_path,
                                                      root_node,
                                                      custom_source_path,
@@ -153,6 +172,7 @@ bool Kinetophone_builder_model::load_from_XML(const string& file_path, Error_par
   // success, update data members
   delete m_slides;
   m_slides = slides;
+  m_slides_source = slides_source;
 
   return_value = true;
   goto exit_point;
