@@ -48,9 +48,14 @@ namespace Roan_trail
     class Kinetophone_builder_model : public Roan_trail::Builder::Movie_builder_model
     {
     public:
+      // enum
+      enum Slides_source_type
+        {
+          directory_source,
+          PDF_source
+        };
       // constructor/destructor
-      Kinetophone_builder_model(const Kinetophone_builder_config& builder_config,
-                                const std::string& slides_source_type);
+      Kinetophone_builder_model(const Kinetophone_builder_config& builder_config);
       ~Kinetophone_builder_model();
       // accessors
       const Roan_trail::Source::Slide_collection<Glib::RefPtr<Gdk::Pixbuf> >* slides() const { return m_slides; }
@@ -62,7 +67,7 @@ namespace Roan_trail
     private:
       const Kinetophone_builder_config& m_builder_config;
       Roan_trail::Source::Slide_collection<Glib::RefPtr<Gdk::Pixbuf> >* m_slides;
-      const std::string m_slides_source_type;
+      Slides_source_type m_slides_source;
       // prevent compiler from generating
       Kinetophone_builder_model(const Kinetophone_builder_model& builder_model);
       Kinetophone_builder_model& operator=(const Kinetophone_builder_model& builder_model);
