@@ -136,7 +136,6 @@ Kinetophone_movie_builder::Kinetophone_movie_builder(const Kinetophone_builder_c
                                   ic_bits_per_sample,
                                   output_size.width,
                                   output_size.height);
-
   if (input_size != output_size)
   {
     Rect_size convert_size;
@@ -179,12 +178,6 @@ const uint8_t* Kinetophone_movie_builder::mf_generate_image(Long_int slide_index
   precondition((slide_index >= 0)
                && (slide_index < m_model.slides()->slide_count()));
 
-  const Slide_collection<RefPtr<Pixbuf> >* slides = m_model.slides();
-  RefPtr<Pixbuf> src_image;
-  const bool image_found = slides->image_for_slide_at(slide_index, src_image);
-  return_used_default = !image_found;
-  const Rect_size src_image_size(src_image->get_width(), src_image->get_height());
-
   const Movie_builder_config& config = *m_config.movie_builder_config;
   const Movie_config& movie_config = *config.movie;
   Rect_size input_size;
@@ -208,6 +201,15 @@ const uint8_t* Kinetophone_movie_builder::mf_generate_image(Long_int slide_index
       assert(false && "invalid movie config for output size");
   }
 
+  const Slide_collection<RefPtr<Pixbuf> >* slides = m_model.slides();
+  RefPtr<Pixbuf> src_image;
+  const bool image_found = slides->image_for_slide_at(slide_index,
+                                                      input_size,
+                                                      src_image);
+  return_used_default = !image_found;
+  const Rect_size src_image_size(src_image->get_width(), src_image->get_height());
+
+
   Rect_origin offset(0, 0);
   Rectangle dest_rect(offset, input_size);
   double scale;
@@ -227,7 +229,13 @@ const uint8_t* Kinetophone_movie_builder::mf_generate_image(Long_int slide_index
   }
   const double scale_x = scale;
   const double scale_y = scale;
-  const int32_t pixel_color = config.movie_attributes.aspect_fill_color.scalar_value();
+  const Color& aspect_fill_color = config.movie_attributes.aspect_fill_color;
+  uint32_t pixel_color = aspect_fill_color.scalar_value();
+  // the fill requires a 32 bit value, adjust if necessary
+  if (!aspect_fill_color.has_alpha)
+  {
+    pixel_color <<= 8;
+  }
   if (input_size == output_size)
   {
     m_pixbuf_cache->fill(pixel_color);
