@@ -29,12 +29,18 @@
 #include <libxml++/nodes/node.h>
 #include <libxml++/parsers/domparser.h>
 #include <boost/lexical_cast.hpp>
-#include <sstream>
+#include <dbus-c++/dbus.h>
+#include <map>
+#include <string>
+#include <vector>
 
+using DBus::Connection;
 using xmlpp::Document;
 using xmlpp::Node;
-using std::stringstream;
 using std::endl;
+using std::string;
+using std::map;
+using std::vector;
 using Roan_trail::Long_int;
 using Roan_trail::Error;
 using Roan_trail::Recorder::Segment;
