@@ -95,7 +95,7 @@ void Kinetophone_dbus_server::create_recorder(string& return_recorder_path,
   return_have_error = false;
 
   start_error_block();
-  handler_error = 0; // avoid unused variable warning
+  static_cast<void>(handler_error); // avoid unused warning
 
   if (m_recorders.size() > 0)
   {
