@@ -39,7 +39,7 @@ namespace Roan_trail
         public DBus::ObjectAdaptor
     {
     public:
-      Kinetophone_dbus_server(DBus::Connection& connection);
+      explicit Kinetophone_dbus_server(DBus::Connection& connection);
       // DBUS interface
       //   control
       void create_recorder(std::string& return_recorder_path,
