@@ -1,6 +1,6 @@
 // Kinetophone_dbus_test_client.cpp
 //
-// Copyright 2011-2012 Roan Trail, Inc.
+// Copyright 2011-2013 Roan Trail, Inc.
 //
 // This file is part of Kinetophone.
 //
@@ -78,10 +78,10 @@ void* client_thread(void *arg)
 
   start_error_block();
 
+  static_cast<void>(handler_error); // avoid unused warning
+
   try
   {
-    handler_error = 0;
-
     recorder->enable_RMS_metering(true);
     recorder->set_metering_channels(1);
 
