@@ -48,15 +48,15 @@ namespace Roan_trail
     {
     public:
       // constructor/destructor
-      Fraction_bar_widget(bool enabled,
-                          bool vertical = false,
-                          double fraction = -1.0,
-                          double warning_fraction = -1.0,
-                          double critical_fraction = -1.0,
-                          double empty_fraction = -1.0,
-                          int bar_thickness = -1,
-                          int bar_width = -1,
-                          int border_padding = 0);
+      explicit Fraction_bar_widget(bool enabled,
+                                   bool vertical = false,
+                                   double fraction = -1.0,
+                                   double warning_fraction = -1.0,
+                                   double critical_fraction = -1.0,
+                                   double empty_fraction = -1.0,
+                                   int bar_thickness = -1,
+                                   int bar_width = -1,
+                                   int border_padding = 0);
       virtual ~Fraction_bar_widget();
       // view update
       virtual bool update(Error_param& return_error);
