@@ -15,6 +15,15 @@
 // General Public License for more details.  You should have received
 // a copy of the GNU General Public License along with Kinetophone. If
 // not, see <http://www.gnu.org/licenses/>.
+//
+// The missing image icon:
+//   kinetophone_missing_image.svn
+// was taken from the Gnome icon theme Dropline Neu! 0.2:
+//   ./scalable/stock/gtk-missing-image.svg
+//   (http://art.gnome.org/themes/icon/1100)
+// which is copyright Silvestre Herrera
+// (silvestre.herrera -at -gmail.com)
+// licensed under the GPL version 2 or later.
 
 #include "Kinetophone_dir_slide_collection.hpp"
 
@@ -23,7 +32,10 @@
 #include <gtkmm/invisible.h>
 #include <gtkmm/stock.h>
 
+#include "../base/Application.hpp"
+#include "../base/common.hpp"
 #include "../base/Dir_slide_collection.hpp"
+#include "../base/Image_types.hpp"
 #include "../base/Slide.hpp"
 #include "../base/File_manager.hpp"
 #include "../base/error/Kinetophone_error.hpp"
@@ -35,6 +47,7 @@ using Gdk::Pixbuf;
 using std::string;
 using std::vector;
 using Roan_trail::Error_param;
+using Roan_trail::max;
 using Roan_trail::Rect_size;
 using Roan_trail::Source::Dir_slide_collection;
 using Roan_trail::Source::Slide;
@@ -47,18 +60,22 @@ using namespace Roan_trail::Kinetophone;
 namespace
 {
   const int ic_thumbnail_size = 64;
+
   RefPtr<Pixbuf> ih_broken_image()
   {
     Gtk::Invisible invisible_widget;
     return invisible_widget.render_icon(Gtk::Stock::MISSING_IMAGE, Gtk::ICON_SIZE_BUTTON);
   }
+  const string ic_missing_image_file = "/kinetophone_missing_image.svg";
 }
 
 //
 // Accessors
 //
 
-bool Kinetophone_dir_slide_collection::image_for_slide_at(int slide_index, RefPtr<Pixbuf>& return_image) const
+bool Kinetophone_dir_slide_collection::image_for_slide_at(int slide_index,
+                                                          const Rect_size& suggested_size,
+                                                          RefPtr<Pixbuf>& return_image) const
 {
   precondition(((slide_index >= 0) && (slide_index < slide_count()))
                && !return_image
@@ -75,7 +92,26 @@ bool Kinetophone_dir_slide_collection::image_for_slide_at(int slide_index, RefPt
   }
   catch (...)
   {
-    return_image = ih_broken_image();
+    const string& installation_dir = Application::application()->installation_dir();
+    try
+    {
+      if ((suggested_size.width <= 0)
+          || (suggested_size.height <= 0))
+      {
+        return_image = Pixbuf::create_from_file(installation_dir + ic_missing_image_file);
+      }
+      else
+      {
+        const int max_dimension = max(suggested_size.width, suggested_size.height);
+        return_image = Pixbuf::create_from_file(installation_dir + ic_missing_image_file,
+                                                max_dimension,
+                                                max_dimension);
+      }
+    }
+    catch (...)
+    {
+      return_image = ih_broken_image();
+    }
   }
 
   postcondition(return_image
@@ -102,7 +138,17 @@ void Kinetophone_dir_slide_collection::thumbnail_for_slide_at(int slide_index,
   }
   catch (...)
   {
-    return_image = ih_broken_image();
+    try
+    {
+      const string& installation_dir = Application::application()->installation_dir();
+      return_image = Pixbuf::create_from_file(installation_dir + ic_missing_image_file,
+                                              ic_thumbnail_size,
+                                              ic_thumbnail_size);
+    }
+    catch (...)
+    {
+      return_image = ih_broken_image();
+    }
   }
 
   postcondition(return_image
@@ -115,7 +161,9 @@ Rect_size Kinetophone_dir_slide_collection::max_bounds() const
   for (int i = 0; i < slide_count(); ++i)
   {
     RefPtr<Pixbuf> image;
-    image_for_slide_at(i, image);
+    image_for_slide_at(i,
+                       Rect_size(0, 0),
+                       image);
     const int image_width = image->get_width();
     if (image_width > bounds.width)
     {
@@ -193,5 +241,6 @@ bool Kinetophone_dir_slide_collection::mf_process_source_images_with_notes(const
   goto exit_point;
 
  exit_point:
+  postcondition(return_error.is_valid_at_return(return_value));
   return return_value;
 }
