@@ -21,6 +21,7 @@
 
 #include <gdkmm/pixbuf.h> // Dir_slide_collection.hpp needs this included first
 #include "../base/Dir_slide_collection.hpp"
+#include "../base/Image_types.hpp"
 #include <string>
 #include <vector>
 
@@ -47,8 +48,11 @@ namespace Roan_trail
       // constructor/destructor
       Kinetophone_dir_slide_collection()
         : Roan_trail::Source::Dir_slide_collection<Glib::RefPtr<Gdk::Pixbuf> >() {}
+      virtual ~Kinetophone_dir_slide_collection() {}
       // accessors
-      virtual bool image_for_slide_at(int slide_index, Glib::RefPtr<Gdk::Pixbuf>& return_image) const;
+      virtual bool image_for_slide_at(int slide_index,
+                                      const Rect_size& suggested_size,
+                                      Glib::RefPtr<Gdk::Pixbuf>& return_image) const;
       virtual void thumbnail_for_slide_at(int slide_index, Glib::RefPtr<Gdk::Pixbuf>& return_image) const;
       virtual Rect_size max_bounds() const;
     protected:
