@@ -87,7 +87,7 @@ namespace
 Kinetophone_narrator_app::Kinetophone_narrator_app(int argc, const char** argv)
   : Application(argc, argv),
     m_config(new Kinetophone_narrator_config),
-    m_model(new Kinetophone_narrator_model("directory"))
+    m_model(0)
 {
   precondition(argv);
 
@@ -213,8 +213,7 @@ bool Kinetophone_narrator_app::mf_invariant(bool check_base_class) const
   bool return_value = false;
 
   // object allocation checking
-  if (!m_config
-      || !m_model)
+  if (!m_config)
   {
     goto exit_point;
   }
@@ -292,6 +291,10 @@ bool Kinetophone_narrator_app::mf_narrate(Error_param& return_error)
 
   bool return_value = false;
 
+  // create the model
+  delete m_model;
+  m_model = new Kinetophone_narrator_model(*m_config);
+
   bool narration_recorder_started_up = false;
   Error_param error;
   Narration_recorder* narration_recorder = m_model->narration_recorder();
@@ -368,8 +371,8 @@ bool Kinetophone_narrator_app::mf_run_GUI(Error_param& return_error)
   Glib::add_exception_handler(&ih_glib_exception_handler);
 
   // load the images
-  const string image_directory = m_config->image_directory;
-  bool slides_loaded = m_model->slides()->load_from_source(image_directory, error);
+  const string image_source_path = m_config->image_source_path;
+  const bool slides_loaded = m_model->slides()->load_from_source(image_source_path, error);
   on_error(!slides_loaded, new Kinetophone_error(error_location(),
                                                  Kinetophone_error::source_load,
                                                  error()));
