@@ -1,6 +1,6 @@
 // Kinetophone_narrator_config.cpp
 //
-// Copyright 2011-2012 Roan Trail, Inc.
+// Copyright 2011-2013 Roan Trail, Inc.
 //
 // This file is part of Kinetophone.
 //
@@ -17,29 +17,39 @@
 // not, see <http://www.gnu.org/licenses/>.
 
 #include "Kinetophone_narrator_config.hpp"
-#include "../base/Sound_recorder_config.hpp"
-#include "../base/Sound_file_config.hpp"
-#include "../base/Level_meter_view.hpp"
+#include "../base/common.hpp"
 #include "../base/File_manager.hpp"
+#include "../base/Image_types.hpp"
+#include "../base/Level_meter_view.hpp"
+#include "../base/option_checks.hpp"
+#include "../base/Option_validators.hpp"
+#include "../base/Sound_file_config.hpp"
+#include "../base/Sound_recorder_config.hpp"
 #include "../base/error/Boost_error.hpp"
 #include "../base/error/Kinetophone_error.hpp"
 #include <boost/program_options.hpp>
-#include <boost/concept/assert.hpp>
-#include <boost/concept_check.hpp>
+#include <exception>
 #include <fstream>
-#include <iostream>
 #include <sstream>
 #include <string>
-#include <vector>
 #include <cstdlib>
 
-using std::stringstream;
-using std::ifstream;
 using std::endl;
-using std::ostream;
+using std::exception;
+using std::ifstream;
+using std::logic_error;
 using std::string;
-using std::vector;
+using std::stringstream;
+using Roan_trail::Color;
 using Roan_trail::File_manager;
+using Roan_trail::Options::check_conflicting_options;
+using Roan_trail::Options::check_option_dependency;
+using Roan_trail::Options::Color_validator;
+using Roan_trail::Options::Directory_validator;
+using Roan_trail::Options::Fraction_validator;
+using Roan_trail::Options::Positive_int_validator;
+using Roan_trail::Options::Str_to_int_lookup_validator;
+using Roan_trail::Options::Validator;
 using Roan_trail::Recorder::Sound_file_config;
 using namespace Roan_trail::Kinetophone;
 
@@ -53,133 +63,9 @@ namespace
 {
   using Roan_trail::Recorder::Sound_recorder_config;
 
-  // option validation support
-  template<class T> struct Ih_validator
-  {
-    BOOST_CONCEPT_ASSERT((boost::CopyConstructible<T>));
-    BOOST_CONCEPT_ASSERT((boost::DefaultConstructible<T>));
-    Ih_validator(const T& a) : v(a) {}
-    Ih_validator() : v(T()) {}
-    T v;
-  };
-
-  template<class T> void validate(boost::any& v, const vector<string>& xs, Ih_validator<T>*, int)
-  {
-    using namespace PO;
-    check_first_occurrence(v);
-    v = boost::any(Ih_validator<T>(boost::lexical_cast<T>(get_single_string(xs))));
-  }
-
-  template<class T> ostream& operator<<(ostream& os, const Ih_validator<T>& c)
-  {
-    return os << c.v;
-  }
-
-  ostream& operator<<(ostream& os, const Ih_validator<bool>& c)
-  {
-    return os << (c.v ? "yes" : "no");
-  }
-
-  void validate(boost::any& v,
-                const std::vector<std::string>& values,
-                Ih_validator<bool>*, int)
-  {
-    using namespace PO;
-
-    check_first_occurrence(v);
-    string s(get_single_string(values, true));
-
-    for (string::size_type i = 0; i < s.size(); ++i)
-    {
-      s[i] = char(tolower(s[i]));
-    }
-
-    if (s.empty() || (s == "on") || (s == "yes") || (s == "1") || (s == "true"))
-    {
-      v = boost::any(Ih_validator<bool>(true));
-    }
-    else if ((s == "off") || (s == "no") || (s == "0") || (s == "false"))
-    {
-      v = boost::any(Ih_validator<bool>(false));
-    }
-    else
-    {
-      throw validation_error(validation_error::invalid_bool_value, s);
-    }
-  }
-
-  typedef bool (*Ih_lookup_ptr)(const string&, int&);
-
-  template<class T, Ih_lookup_ptr F> struct Ih_lookup_validator
-  {
-    BOOST_CONCEPT_ASSERT((boost::CopyConstructible<T>));
-    Ih_lookup_validator(const T& a) : v(a) {}
-    T v;
-  };
-
-  template<class T, Ih_lookup_ptr F> ostream& operator<<(ostream& os,
-                                                         const Ih_lookup_validator<T, F>& c)
-  {
-    return os << c.v;
-  }
-
-  template<class T, Ih_lookup_ptr F> void validate(boost::any& v,
-                const std::vector<std::string>& values,
-                Ih_lookup_validator<T, F>*, int)
-  {
-    using namespace PO;
-
-    validators::check_first_occurrence(v);
-    // error if more than one string supplied
-    const string& s = validators::get_single_string(values);
-
-    int type;
-    if (F(s, type))
-    {
-      v = boost::any(Ih_lookup_validator<T, F>(s));
-    }
-    else
-    {
-      throw validation_error(validation_error::invalid_option_value);
-    }
-  }
-
-  template<class T, int i> struct Ih_cond_validator
-  {
-    BOOST_CONCEPT_ASSERT((boost::CopyConstructible<T>));
-    BOOST_CONCEPT_ASSERT((boost::DefaultConstructible<T>));
-    Ih_cond_validator(const T& a) : v(a) {}
-    Ih_cond_validator() : v(T()) {}
-    bool cond() const;
-    T v;
-  };
-
-  template<class T, int i> ostream& operator<<(ostream& os, const Ih_cond_validator<T, i>& c)
-  {
-    return os << c.v;
-  }
-
-  template<class T, int i> void validate(boost::any& v,
-                                         const std::vector<std::string>& values,
-                                         Ih_cond_validator<T, i>*, int)
-  {
-    using namespace PO;
-
-    check_first_occurrence(v);
-    string s(get_single_string(values, true));
-
-    T a = boost::lexical_cast<T>(s);
-    Ih_cond_validator<T, i> cv(a);
-
-    if (cv.cond())
-    {
-      v = boost::any(cv);
-    }
-    else
-    {
-      throw validation_error(validation_error::invalid_option_value);
-    }
-  }
+  //
+  // some custom validation code for this configuration
+  //
 
   bool ih_filter_file_type_string(const string& file_type_string, int& check_file_type)
   {
@@ -228,39 +114,37 @@ namespace
     return return_value;
   }
 
-  typedef struct Ih_lookup_validator<string, ih_filter_file_type_string>
-  Ih_file_type_validator;
-  typedef struct Ih_lookup_validator<string, ih_filter_data_format_string>
-  Ih_data_format_validator;
-  typedef struct Ih_lookup_validator<string, Sound_file_config::endianness_for_string>
-  Ih_endianness_validator;
-  typedef struct Ih_cond_validator<int, 0> Ih_pos_int_validator;
-  template<> bool Ih_pos_int_validator::cond() const
-  {
-    return v > 0;
-  }
-  typedef struct Ih_cond_validator<double, 0> Ih_fraction_validator;
-  template<> bool Ih_fraction_validator::cond() const
-  {
-    return ((v >= 0.0) && (v <= 1.0));
-  }
+  typedef struct Str_to_int_lookup_validator<string,
+                                             ih_filter_file_type_string> Ih_file_type_validator;
+  typedef struct Str_to_int_lookup_validator<string,
+                                             ih_filter_data_format_string> Ih_data_format_validator;
+  typedef struct Str_to_int_lookup_validator<string,
+                                             Sound_file_config::endianness_for_string> Ih_endianness_validator;
+
+  //
+  // the main validator struct for this configuration
+  //
 
   struct Ih_validator_struct
   {
     Ih_validator_struct(const Kinetophone_narrator_config& s)
-      : image_directory(s.image_directory),
+      : image_source_path(s.image_source_path),
+        image_directory(),
+        PDF_fill_color(Color_validator(*s.PDF_fill_color)),
+        import_PDF_notes(Validator<bool>(s.import_PDF_notes)),
         session_file_path(s.session_file_path),
         level_meter(s.level_meter),
-        update_rate(Ih_pos_int_validator(s.update_rate)),
-        show_file_size(Ih_validator<bool>(s.show_file_size)),
-        show_audio_info(Ih_validator<bool>(s.show_audio_info)),
-        show_overflow_count(Ih_validator<bool>(s.show_overflow_count)),
-        detailed_error(Ih_validator<bool>(s.detailed_error)),
-        custom_installation_dir(s.custom_installation_dir),
+        update_rate(Positive_int_validator(s.update_rate)),
+        show_file_size(Validator<bool>(s.show_file_size)),
+        show_audio_info(Validator<bool>(s.show_audio_info)),
+        show_overflow_count(Validator<bool>(s.show_overflow_count)),
+        detailed_error(Validator<bool>(s.detailed_error)),
+        custom_installation_dir(Directory_validator(s.custom_installation_dir)),
         sound_recorder_config(*s.sound_recorder_config) {}
     void update(Kinetophone_narrator_config& return_cfg)
     {
-      return_cfg.image_directory = image_directory;
+      *return_cfg.PDF_fill_color = PDF_fill_color.v;
+      return_cfg.import_PDF_notes = import_PDF_notes.v;
       return_cfg.session_file_path = session_file_path;
       level_meter.update(return_cfg.level_meter);
       return_cfg.update_rate = update_rate.v;
@@ -268,9 +152,25 @@ namespace
       return_cfg.show_audio_info = show_audio_info.v;
       return_cfg.show_overflow_count = show_overflow_count.v;
       return_cfg.detailed_error = detailed_error.v;
-      return_cfg.custom_installation_dir = custom_installation_dir;
+      return_cfg.custom_installation_dir = custom_installation_dir.v;
       sound_recorder_config.update(*return_cfg.sound_recorder_config);
       // calculate and assign remaining members
+      if (("" == image_source_path))
+      {
+        if ("" != image_directory.v)
+        {
+          return_cfg.image_source_path = image_directory.v;
+        }
+        else
+        {
+          // if not specified, use the current directory for the image source path
+          return_cfg.image_source_path = ".";
+        }
+      }
+      else
+      {
+        return_cfg.image_source_path = image_source_path;
+      }
       string use_file_type = sound_recorder_config.sound_file.file_type.v;
       if ("" == use_file_type)
       {
@@ -304,15 +204,18 @@ namespace
         return_cfg.sound_recorder_config->sound_file->endianness = Sound_file_config::default_endianness;
       }
     }
-    string image_directory;
+    string image_source_path;
+    Directory_validator image_directory;
+    Color_validator PDF_fill_color;
+    Validator<bool> import_PDF_notes;
     string session_file_path;
     struct Level_meter_validator_struct
     {
       Level_meter_validator_struct(const Kinetophone_narrator_config::Level_meter_struct& s)
-        : model_ballistics(Ih_validator<bool>(s.model_ballistics)),
-          attack_period(Ih_pos_int_validator(s.attack_period)),
-          decay_period(Ih_pos_int_validator(s.decay_period)),
-          peak_hold_period(Ih_pos_int_validator(s.peak_hold_period)) {}
+        : model_ballistics(Validator<bool>(s.model_ballistics)),
+          attack_period(Positive_int_validator(s.attack_period)),
+          decay_period(Positive_int_validator(s.decay_period)),
+          peak_hold_period(Positive_int_validator(s.peak_hold_period)) {}
       void update(Kinetophone_narrator_config::Level_meter_struct& return_s)
       {
         return_s.model_ballistics = model_ballistics.v;
@@ -321,24 +224,24 @@ namespace
         return_s.peak_hold_period = peak_hold_period.v;
       }
       //
-      Ih_validator<bool> model_ballistics;
-      Ih_pos_int_validator attack_period;
-      Ih_pos_int_validator decay_period;
-      Ih_pos_int_validator peak_hold_period;
+      Validator<bool> model_ballistics;
+      Positive_int_validator attack_period;
+      Positive_int_validator decay_period;
+      Positive_int_validator peak_hold_period;
     } level_meter;
-    Ih_pos_int_validator update_rate;
-    Ih_validator<bool> show_file_size;
-    Ih_validator<bool> show_audio_info;
-    Ih_validator<bool> show_overflow_count;
-    Ih_validator<bool> detailed_error;
-    string custom_installation_dir;
+    Positive_int_validator update_rate;
+    Validator<bool> show_file_size;
+    Validator<bool> show_audio_info;
+    Validator<bool> show_overflow_count;
+    Validator<bool> detailed_error;
+    Directory_validator custom_installation_dir;
     struct Sound_recorder_config_validator_struct
     {
       Sound_recorder_config_validator_struct(const Sound_recorder_config& s)
         : input_device(s.input_device),
-          frames_per_buffer(Ih_pos_int_validator(s.frames_per_buffer)),
-          write_buffer_factor(Ih_pos_int_validator(s.write_buffer_factor)),
-          RMS_level_integration_period(Ih_pos_int_validator(s.RMS_level_integration_period)),
+          frames_per_buffer(Positive_int_validator(s.frames_per_buffer)),
+          write_buffer_factor(Positive_int_validator(s.write_buffer_factor)),
+          RMS_level_integration_period(Positive_int_validator(s.RMS_level_integration_period)),
           sound_file(*s.sound_file),
           file_overwrite(s.file_overwrite) {}
       void update(Sound_recorder_config& return_s)
@@ -352,15 +255,15 @@ namespace
       }
       //
       int input_device;
-      Ih_pos_int_validator frames_per_buffer;
-      Ih_pos_int_validator write_buffer_factor;
-      Ih_pos_int_validator RMS_level_integration_period;
+      Positive_int_validator frames_per_buffer;
+      Positive_int_validator write_buffer_factor;
+      Positive_int_validator RMS_level_integration_period;
       struct Sound_file_config_validator_struct
       {
         Sound_file_config_validator_struct(const Sound_file_config& s)
-          : channels(Ih_pos_int_validator(s.channels)),
-            sample_rate(Ih_pos_int_validator(static_cast<int>(s.sample_rate))),
-            VBR_quality(Ih_fraction_validator(s.VBR_quality)),
+          : channels(Positive_int_validator(s.channels)),
+            sample_rate(Positive_int_validator(static_cast<int>(s.sample_rate))),
+            VBR_quality(Fraction_validator(s.VBR_quality)),
             file_name(s.file_name),
             file_type(Ih_file_type_validator("")),
             data_format(Ih_data_format_validator(Sound_file_config::string_for_data_format(s.data_format))),
@@ -385,94 +288,49 @@ namespace
           }
         }
         //
-        Ih_pos_int_validator channels;
-        Ih_pos_int_validator sample_rate;
-        Ih_fraction_validator VBR_quality;
+        Positive_int_validator channels;
+        Positive_int_validator sample_rate;
+        Fraction_validator VBR_quality;
         string file_name;
         Ih_file_type_validator file_type;
         Ih_data_format_validator data_format;
         Ih_endianness_validator endianness;
       } sound_file;
-      Ih_validator<bool> file_overwrite;
+      Validator<bool> file_overwrite;
     } sound_recorder_config;
   };
 
-  //
-  // Auxiliary functions for checking input validity.
-  //
-
-  // Function used to check that 'opt1' and 'opt2' are not specified
-  // at the same time.
-  void ih_check_conflicting_options(const PO::variables_map& VM,
-                                    const char* opt1,
-                                    const char* opt2,
-                                    bool condition = true,
-                                    const string& message = "")
-  {
-    precondition(opt1
-                 && opt2);
-
-    if (condition &&
-        (VM.count(opt1) && !VM[opt1].defaulted()
-         && VM.count(opt2) && !VM[opt2].defaulted()))
-    {
-      string exception_message = string("conflicting options '")
-        + opt1 + "' and '" + opt2 + "'";
-      if ("" != message)
-      {
-        exception_message += string(": ") + message;
-      }
-      exception_message += ".";
-      throw std::logic_error(exception_message);
-    }
-  }
-
-  // function used to check that 'for_what' is specified, then
-  // 'required_option' is specified too.
-  void ih_check_option_dependency(const PO::variables_map& VM,
-                                  const char* for_what,
-                                  const char* required_option,
-                                  bool condition = true)
-  {
-    precondition(for_what
-                 && required_option);
-
-    if (VM.count(for_what) && !VM[for_what].defaulted())
-    {
-      if ((VM.count(required_option) == 0)
-          || VM[required_option].defaulted()
-          || !condition)
-      {
-        throw std::logic_error(string("option '") + for_what
-                               + "' requires option '" + required_option + "'.");
-      }
-    }
-  }
-
   void ih_check_option_consistency(const Kinetophone_narrator_config& cfg,
                                    const PO::variables_map& VM)
   {
     // conflicting option checks
-    ih_check_conflicting_options(VM,
-                                 "file-type",
-                                 "overwrite",
-                                 (cfg.sound_recorder_config->file_overwrite
-                                  && (Sound_file_config::file_type_flac
-                                      == cfg.sound_recorder_config->sound_file->file_type)),
-                                 "FLAC file format cannot be specified with file overwrite");
+    check_conflicting_options(VM,
+                              "file-type",
+                              "overwrite",
+                              (cfg.sound_recorder_config->file_overwrite
+                               && (Sound_file_config::file_type_flac
+                                   == cfg.sound_recorder_config->sound_file->file_type)),
+                              "FLAC file format cannot be specified with file overwrite");
+    check_conflicting_options(VM,
+                              "image-directory",
+                              "image-source-path");
+    check_conflicting_options(VM,
+                              "import-notes-from-file",
+                              "import-PDF-notes",
+                              cfg.import_PDF_notes);
     // option dependency checks
-    ih_check_option_dependency(VM,
-                               "level-meter-attack-period",
-                               "level-meter-ballistics",
-                               cfg.level_meter.model_ballistics);
-    ih_check_option_dependency(VM,
-                               "level-meter-decay-period",
-                               "level-meter-ballistics",
-                               cfg.level_meter.model_ballistics);
+    check_option_dependency(VM,
+                            "level-meter-attack-period",
+                            "level-meter-ballistics",
+                            cfg.level_meter.model_ballistics);
+    check_option_dependency(VM,
+                            "level-meter-decay-period",
+                            "level-meter-ballistics",
+                            cfg.level_meter.model_ballistics);
     // invalid file type
     if (Sound_file_config::file_type_unknown == cfg.sound_recorder_config->sound_file->file_type)
     {
-      throw std::logic_error("could not determine a valid sound file type");
+      throw logic_error("could not determine a valid sound file type");
     }
   }
 
@@ -501,32 +359,33 @@ namespace
       ;
 
     return_desc_params.basic_desc.add_options()
-      ("audio-info,I", PO::value<Ih_validator<bool> >(&return_v.show_audio_info)
-       ->implicit_value(Ih_validator<bool>(true)),
+      ("audio-info,I", PO::value<Validator<bool> >(&return_v.show_audio_info)
+       ->implicit_value(Validator<bool>(true)),
        "show audio information when recording")
       ("device,d", PO::value<int>(&return_v.sound_recorder_config.input_device)
        ->default_value(return_v.sound_recorder_config.input_device),
        "set input device number (from --list-devices option)")
-      ("image-directory,D", PO::value<string>(&return_v.image_directory)
-       ->default_value(return_v.image_directory),
+      ("image-directory,D", PO::value<Directory_validator>(&return_v.image_directory),
        "set image source directory")
+      ("image-source-path,S", PO::value<string>(&return_v.image_source_path)
+       ->default_value(return_v.image_source_path),
+       "set image source path (directory or PDF file)")
       ("import-notes-from-file,n", PO::value<string>(&return_v.session_file_path)
        ->default_value(return_v.session_file_path),
-       "import notes from session XML file")
+       "import notes from session XML file (cannot use with --import-PDF-notes yes")
       ("sound-file,f", PO::value<string>(&return_v.sound_recorder_config.sound_file.file_name)
        ->default_value(return_v.sound_recorder_config.sound_file.file_name),
        "set destination sound file path")
-      ("overwrite,o", PO::value<Ih_validator<bool> >(&return_v.sound_recorder_config.file_overwrite)
-       ->implicit_value(Ih_validator<bool>(true)),
+      ("overwrite,o", PO::value<Validator<bool> >(&return_v.sound_recorder_config.file_overwrite)
+       ->implicit_value(Validator<bool>(true)),
        "overwrite sound recording file")
-      ("channels,c", PO::value<Ih_pos_int_validator>(&return_v.sound_recorder_config.sound_file.channels)
+      ("channels,c", PO::value<Positive_int_validator>(&return_v.sound_recorder_config.sound_file.channels)
        ->default_value(return_v.sound_recorder_config.sound_file.channels),
        "set channels to record (must be > 0)")
-      ("sample-rate,s", PO::value<Ih_pos_int_validator>(&return_v.sound_recorder_config.sound_file.sample_rate)
+      ("sample-rate,s", PO::value<Positive_int_validator>(&return_v.sound_recorder_config.sound_file.sample_rate)
        ->default_value(return_v.sound_recorder_config.sound_file.sample_rate),
        "set samples per second (must be > 0)")
-      ("file-type,t", PO::value<Ih_file_type_validator>(&return_v.sound_recorder_config.sound_file.file_type)
-       ->default_value(return_v.sound_recorder_config.sound_file.file_type),
+      ("file-type,t", PO::value<Ih_file_type_validator>(&return_v.sound_recorder_config.sound_file.file_type),
        "set audio file type\n"
        "{ wav | aiff | flac }") // (check_code_ignore)
       ("data-format,a", PO::value<Ih_data_format_validator>(&return_v.sound_recorder_config.sound_file
@@ -534,9 +393,16 @@ namespace
        ->default_value(return_v.sound_recorder_config.sound_file.data_format),
        "set format of samples in the audio file (signed values)\n"
        "{ pcm8 | pcm16 | pcm24 | pcm32 | float | double }")
-      ("show-file-size,F", PO::value<Ih_validator<bool> >(&return_v.show_file_size)
-       ->implicit_value(Ih_validator<bool>(true)),
+      ("show-file-size,F", PO::value<Validator<bool> >(&return_v.show_file_size)
+       ->implicit_value(Validator<bool>(true)),
        "display the size of the output file")
+      ("PDF-fill-color,L",
+       PO::value<Color_validator>(&return_v.PDF_fill_color)
+       ->default_value(return_v.PDF_fill_color),
+       "set fill color for PDF background (format RRGGBB, in hexadecimal")
+      ("import-PDF-notes,N", PO::value<Validator<bool> >(&return_v.import_PDF_notes)
+       ->default_value(return_v.import_PDF_notes),
+       "import PDF text annotations as slide notes (value cannot be yes with --import-notes-from-file)")
       ;
 
     return_desc_params.advanced_desc.add_options()
@@ -544,45 +410,44 @@ namespace
        ->default_value(return_v.sound_recorder_config.sound_file.endianness),
        "set endian order in the output file\n"
        "\t{ file | little | big | cpu }")
-      ("frames-per-buffer,b", PO::value<Ih_pos_int_validator>(&return_v.sound_recorder_config.frames_per_buffer)
+      ("frames-per-buffer,b", PO::value<Positive_int_validator>(&return_v.sound_recorder_config.frames_per_buffer)
        ->default_value(return_v.sound_recorder_config.frames_per_buffer),
        "set number of samples processed at a time (must be > 0)")
       ("write-buffer-factor,W",
-       PO::value<Ih_pos_int_validator>(&return_v.sound_recorder_config.write_buffer_factor)
+       PO::value<Positive_int_validator>(&return_v.sound_recorder_config.write_buffer_factor)
        ->default_value(return_v.sound_recorder_config.write_buffer_factor),
        "set factor to multiply frames per buffer for sizing write buffer (must be > 0)")
       ("RMS-integration-period,R",
-       PO::value<Ih_pos_int_validator>(&return_v.sound_recorder_config.RMS_level_integration_period)
+       PO::value<Positive_int_validator>(&return_v.sound_recorder_config.RMS_level_integration_period)
        ->default_value(return_v.sound_recorder_config.RMS_level_integration_period),
        "set the period in milliseconds used for RMS level averaging (must be > 0)")
-      ("update-rate,U", PO::value<Ih_pos_int_validator>(&return_v.update_rate)
+      ("update-rate,U", PO::value<Positive_int_validator>(&return_v.update_rate)
        ->default_value(return_v.update_rate),
        "set number of times display is refreshed per second (must be > 0)")
-      ("level-meter-ballistics,B", PO::value<Ih_validator<bool> >(&return_v.level_meter.model_ballistics)
-       ->implicit_value(Ih_validator<bool>(return_v.level_meter.model_ballistics)),
+      ("level-meter-ballistics,B", PO::value<Validator<bool> >(&return_v.level_meter.model_ballistics)
+       ->implicit_value(Validator<bool>(return_v.level_meter.model_ballistics)),
        "model ballistics in level meter (requires --level-meter or -m)")
-      ("level-meter-attack-period,P", PO::value<Ih_pos_int_validator>(&return_v.level_meter.attack_period)
+      ("level-meter-attack-period,P", PO::value<Positive_int_validator>(&return_v.level_meter.attack_period)
        ->default_value(return_v.level_meter.attack_period),
        "set the number of milliseconds for the level meter to travel forward 0.99X full scale (must be > 0, "
        "requires --level-meter-ballistics)")
-      ("level-meter-decay-period,p", PO::value<Ih_pos_int_validator>(&return_v.level_meter.decay_period)
+      ("level-meter-decay-period,p", PO::value<Positive_int_validator>(&return_v.level_meter.decay_period)
        ->default_value(return_v.level_meter.decay_period),
        "set the number of milliseconds for the level meter to decay back from 0.99X full scale (must be > 0, "
        "requires --level-meter-ballistics)")
-      ("level-meter-peak-hold-period,H", PO::value<Ih_pos_int_validator>(&return_v.level_meter.peak_hold_period)
+      ("level-meter-peak-hold-period,H", PO::value<Positive_int_validator>(&return_v.level_meter.peak_hold_period)
        ->default_value(return_v.level_meter.peak_hold_period),
        "set the number of milliseconds for the level meter to maintain peak (must be > 0)")
-      ("detailed-error,e", PO::value<Ih_validator<bool> >(&return_v.detailed_error)
-       ->implicit_value(Ih_validator<bool>(true)),
+      ("detailed-error,e", PO::value<Validator<bool> >(&return_v.detailed_error)
+       ->implicit_value(Validator<bool>(true)),
        "display detailed diagnostics on error")
-      ("overflow-count,O", PO::value<Ih_validator<bool> >(&return_v.show_overflow_count)
-       ->implicit_value(Ih_validator<bool>(true)),
+      ("overflow-count,O", PO::value<Validator<bool> >(&return_v.show_overflow_count)
+       ->implicit_value(Validator<bool>(true)),
        "show input/output overflow count")
-      ("VBR-quality,q", PO::value<Ih_fraction_validator>(&return_v.sound_recorder_config.sound_file.VBR_quality)
+      ("VBR-quality,q", PO::value<Fraction_validator>(&return_v.sound_recorder_config.sound_file.VBR_quality)
        ->default_value(Sound_file_config::highest_VBR_quality),
        "set variable bit rate (VBR) quality (0.0 = lowest to 1.0 = highest)")
-      ("config-directory,C", PO::value<string>(&return_v.custom_installation_dir)
-       ->default_value(return_v.custom_installation_dir),
+      ("config-directory,C", PO::value<Directory_validator>(&return_v.custom_installation_dir),
        "set custom program configuration directory")
       ;
   }
@@ -595,7 +460,9 @@ namespace
 Kinetophone_narrator_config::Kinetophone_narrator_config()
   // setup some reasonable defaults
   : help_message(""),
-    image_directory("."),
+    image_source_path(""),
+    PDF_fill_color(new Color(default_PDF_fill_color, false)),
+    import_PDF_notes(true),
     session_file_path(),
     level_meter(Level_meter_struct()),
     update_rate(default_update_rate),
@@ -611,6 +478,7 @@ Kinetophone_narrator_config::Kinetophone_narrator_config()
 Kinetophone_narrator_config::~Kinetophone_narrator_config()
 {
   delete sound_recorder_config;
+  delete PDF_fill_color;
 }
 
 Kinetophone_narrator_config::Level_meter_struct::Level_meter_struct()
@@ -628,6 +496,7 @@ Kinetophone_narrator_config::Level_meter_struct::Level_meter_struct()
 
 //   default
 const int Kinetophone_narrator_config::default_update_rate = 30;
+const uint32_t Kinetophone_narrator_config::default_PDF_fill_color = 0xFFFFFFFF; // white
 //   command
 const int Kinetophone_narrator_config::command_error;
 const int Kinetophone_narrator_config::command_narrate;
@@ -680,7 +549,8 @@ int Kinetophone_narrator_config::parse_program_options(int argc,
     help_stream << "Usage: " << endl;
     help_stream << "kinetophone_narrator --help" << endl << endl;
     help_stream << "kinetophone_narrator --list-devices" << endl << endl;
-    help_stream << "kinetophone_narrator [image_directory [sound_file ]] [ options ]" << endl << endl;
+    help_stream << "kinetophone_narrator [image-directory [sound_file ]] [ options ]" << endl << endl;
+    help_stream << "kinetophone_narrator [PDF-file-path [sound_file ]] [ options ]" << endl << endl;
 
     PO::options_description all_desc("Options");
     all_desc.add(cmd_line_desc).add(basic_desc).add(advanced_desc);
@@ -691,7 +561,7 @@ int Kinetophone_narrator_config::parse_program_options(int argc,
 
     // parse the command line (overrides config file)
     PO::positional_options_description pos_desc;
-    pos_desc.add("image-directory", 1);
+    pos_desc.add("image-source-path", 1);
     pos_desc.add("sound-file", 1);
     PO::store(PO::command_line_parser(argc, const_cast<char**>(argv)).
               options(all_desc).positional(pos_desc).run(), VM);
@@ -729,7 +599,7 @@ int Kinetophone_narrator_config::parse_program_options(int argc,
       return_value = Kinetophone_narrator_config::command_narrate;
     }
   }
-  catch (const std::exception& e)
+  catch (const exception& e)
   {
     string diagnostic = e.what();
     stringstream s;
