@@ -19,10 +19,12 @@
 #ifndef KINETOPHONE_NARRATOR_CONFIG_HPP_
 #define KINETOPHONE_NARRATOR_CONFIG_HPP_
 
+#include "../base/common.hpp"
 #include <string>
 
 namespace Roan_trail
 {
+  class Color;
   class Error_param;
 
   namespace Recorder
@@ -46,7 +48,9 @@ namespace Roan_trail
       // typedef
       // structure members
       std::string help_message;
-      std::string image_directory;
+      std::string image_source_path;
+      Roan_trail::Color* PDF_fill_color;
+      bool import_PDF_notes;
       std::string session_file_path;
       struct Level_meter_struct
       {
@@ -65,6 +69,7 @@ namespace Roan_trail
       Roan_trail::Recorder::Sound_recorder_config* sound_recorder_config;
       // defaults
       static const int default_update_rate;
+      static const uint32_t default_PDF_fill_color;
       // command class constants
       static const int command_error          = 0;
       static const int command_narrate        = 1;
