@@ -21,8 +21,12 @@
 // include *mm first to avoid conflicts
 #include <gdkmm/pixbuf.h>
 
+#include "Kinetophone_narrator_config.hpp"
 #include "Kinetophone_dir_slide_collection.hpp"
+#include "Kinetophone_PDF_slide_collection.hpp"
+#include "../base/File_manager.hpp"
 #include "../base/Narration_recorder.hpp"
+#include "../base/Slide_collection.hpp"
 #include "../base/error/Kinetophone_error.hpp"
 #include "../base/error/Posix_error.hpp"
 #include <libxml++/document.h>
@@ -40,27 +44,46 @@ using std::ios_base;
 using std::ofstream;
 using std::string;
 using Roan_trail::Error_param;
+using Roan_trail::File_manager;
 using Roan_trail::Recorder::Narration_recorder;
+using Roan_trail::Source::Slide_collection;
+using Roan_trail::Source::source_type_from_XML;
 using namespace Roan_trail::Kinetophone;
 
 //
 // Constructor/destructor
 //
 
-Kinetophone_narrator_model::Kinetophone_narrator_model(const string& slides_source_type)
+Kinetophone_narrator_model::Kinetophone_narrator_model(const Kinetophone_narrator_config& config)
   : m_slides(0),
+    m_slides_source(directory_source),
     m_recorder(new Narration_recorder),
     m_dirty(false)
 {
-  if ("directory" == slides_source_type)
-  {
-    m_slides = new Kinetophone_dir_slide_collection;
-  }
-  else
+  m_slides_source = (File_manager::path_is_directory(config.image_source_path) ?
+                     directory_source : PDF_source);
+
+  switch (m_slides_source)
   {
+  case directory_source:
+    {
+      m_slides = new Kinetophone_dir_slide_collection;
+    }
+    break;
+  case PDF_source:
+    {
+      Kinetophone_PDF_slide_collection* slides = new Kinetophone_PDF_slide_collection;
+      slides->set_import_PDF_notes(config.import_PDF_notes);
+      slides->set_fill_color(*config.PDF_fill_color);
+      m_slides = slides;
+    }
+    break;
+  default:
     assert(false && "invalid slides source type when creating Kinetophone narrator model");
+    break;
   }
 
+
   postcondition(mf_invariant(false));
 }
 
@@ -101,9 +124,11 @@ bool Kinetophone_narrator_model::import_from_XML(const string& file_path, Error_
                && mf_invariant());
 
   bool return_value = false;
+
   start_error_block();
 
   Error_param error;
+  string diagnostic_prefix = "error importing XML file: ";
 
   try
   {
@@ -131,7 +156,7 @@ bool Kinetophone_narrator_model::import_from_XML(const string& file_path, Error_
   }
   catch (const xmlpp::exception& e)
   {
-    const string diagnostic = string("error importing XML file: ") + e.what();
+    const string diagnostic = diagnostic_prefix + e.what();
     on_error(true, new Kinetophone_error(error_location(),
                                          Kinetophone_error::file_IO,
                                          diagnostic,
@@ -143,13 +168,13 @@ bool Kinetophone_narrator_model::import_from_XML(const string& file_path, Error_
   goto exit_point;
   end_error_block();
 
-  default_error_handler_and_cleanup(return_error,
-                                    return_value,
-                                    false);
+  default_error_handler_and_cleanup(return_error, return_value, false);
+
   goto exit_point;
 
  exit_point:
-  postcondition(mf_invariant());
+  postcondition(return_error.is_valid_at_return(return_value)
+                && mf_invariant());
   return return_value;
 }
 
