@@ -46,11 +46,19 @@ namespace Roan_trail
 
   namespace Kinetophone
   {
+    class Kinetophone_narrator_config;
+
     class Kinetophone_narrator_model
     {
     public:
+      // enum
+      enum Slides_source_type
+        {
+          directory_source,
+          PDF_source
+        };
       // constructor/destructor
-      Kinetophone_narrator_model(const std::string& slides_source_type);
+      explicit Kinetophone_narrator_model(const Kinetophone_narrator_config& config);
       ~Kinetophone_narrator_model();
       // accessors
       const Roan_trail::Source::Slide_collection<Glib::RefPtr<Gdk::Pixbuf> >* slides() const { return m_slides; }
@@ -68,6 +76,7 @@ namespace Roan_trail
       bool mf_invariant(bool check_base_class = true) const;
     private:
       Roan_trail::Source::Slide_collection<Glib::RefPtr<Gdk::Pixbuf> >* m_slides;
+      Slides_source_type m_slides_source;
       Roan_trail::Recorder::Narration_recorder* m_recorder;
       bool m_dirty;
       // prevent compiler from generating
