@@ -204,7 +204,11 @@ void Kinetophone_narrator_view::model_slides_updated()
 
   // update slide image
   RefPtr<Pixbuf> image;
-  slides->image_for_slide_at(slide_index, image);
+  const Gdk::Rectangle& image_rect = m_main_window->slide_image_widget()->image_rect();
+  const Rect_size image_size(image_rect.get_width(), image_rect.get_height());
+  slides->image_for_slide_at(slide_index,
+                             image_size,
+                             image);
   m_main_window->image_aspectframe()->set(Gtk::ALIGN_CENTER,
                                           Gtk::ALIGN_CENTER,
                                           static_cast<float>(image->get_width())
