@@ -61,18 +61,17 @@
 #include <gtkmm/window.h>
 #include <pangomm/fontdescription.h>
 
-#include "../base/common.hpp"
 #include "Fraction_bar_widget.hpp"
 #include "Level_meter_widget.hpp"
 #include "Scaled_image_widget.hpp"
 #include "Kinetophone_narrator_config.hpp"
+#include "../base/Application.hpp"
+#include "../base/common.hpp"
 #include "../base/Sound_recorder_config.hpp"
 #include "../base/Sound_file_config.hpp"
 #include "../base/error/Kinetophone_error.hpp"
-#include "../base/Application.hpp"
 #include <string>
 
-using Gdk::Color;
 using Glib::RefPtr;
 using Gtk::AccelGroup;
 using Gtk::Alignment;
@@ -233,7 +232,7 @@ bool Kinetophone_narrator_window::mf_invariant(bool check_base_class) const
 void Kinetophone_narrator_window::mf_setup(const Kinetophone_narrator_config& config)
 {
   // setup main window
-  set_title(string("Kinetophone is narrating: ") + config.image_directory);
+  set_title(string("Kinetophone is narrating: ") + config.image_source_path);
   set_default_size(ic_default_main_window_width, ic_default_main_window_height);
   const string& installation_dir = Application::application()->installation_dir();
   try
@@ -393,7 +392,7 @@ Widget* Kinetophone_narrator_window::mf_setup_control_toolbar()
   ToolItem* time_tool_item = manage(new ToolItem);
   time_tool_item->set_is_important();
   control_toolbar->append(*time_tool_item);
-  Color LCD_green;
+  Gdk::Color LCD_green;
   LCD_green.set_rgb(0.80 * 65535, 0.81 * 65535, 0.68 * 65535);
   EventBox* record_time_event_box = manage(new EventBox());
   record_time_event_box->modify_bg(Gtk::STATE_NORMAL, LCD_green);
