@@ -54,7 +54,7 @@ namespace Roan_trail
     {
     public:
       // constructor/destructor/copy
-      Kinetophone_narrator_window(const Kinetophone_narrator_config& config);
+      explicit Kinetophone_narrator_window(const Kinetophone_narrator_config& config);
       ~Kinetophone_narrator_window();
       // commands
       void setup() { m_setup = true; }
