@@ -54,7 +54,8 @@ namespace Roan_trail
       virtual ~Scaled_image_widget();
       // view update
       virtual bool update(Error_param& return_error);
-      // mutators
+      // accessors/mutators
+      const Gdk::Rectangle& image_rect() const { return m_image_rect; };
       void set_image(const Glib::RefPtr<Gdk::Pixbuf>& image);
     protected:
       // invariant check
