@@ -1,6 +1,6 @@
 // Kinetophone_vox_config.cpp
 //
-// Copyright 2011-2012 Roan Trail, Inc.
+// Copyright 2011-2013 Roan Trail, Inc.
 //
 // This file is part of Kinetophone.
 //
@@ -20,28 +20,34 @@
 #include "../base/Speech_synthesizer_config.hpp"
 #include "../base/Sound_file_config.hpp"
 #include "../base/File_manager.hpp"
+#include "../base/option_checks.hpp"
+#include "../base/Option_validators.hpp"
 #include "../base/error/Boost_error.hpp"
 #include "../base/error/Kinetophone_error.hpp"
 #include <boost/program_options.hpp>
 #include <boost/concept/assert.hpp>
 #include <boost/concept_check.hpp>
 #include <fstream>
-#include <iostream>
 #include <sstream>
 #include <string>
-#include <vector>
 #include <cstdlib>
 
 using std::endl;
 using std::ifstream;
+using std::logic_error;
 using std::string;
 using std::stringstream;
-using std::ostream;
-using std::vector;
 using Roan_trail::File_manager;
+using Roan_trail::Options::check_conflicting_options;
+using Roan_trail::Options::check_option_dependency;
+using Roan_trail::Options::Directory_validator;
+using Roan_trail::Options::Positive_int_validator;
+using Roan_trail::Options::Str_to_str_lookup_validator;
+using Roan_trail::Options::Validator;
 using Roan_trail::Recorder::Sound_file_config;
 using Roan_trail::Recorder::Speech_synthesizer_config;
 using namespace Roan_trail::Kinetophone;
+
 namespace PO = boost::program_options;
 
 //
@@ -50,181 +56,72 @@ namespace PO = boost::program_options;
 
 namespace
 {
-  // option validation support
-  template<class T> struct Ih_validator
-  {
-    BOOST_CONCEPT_ASSERT((boost::CopyConstructible<T>));
-    BOOST_CONCEPT_ASSERT((boost::DefaultConstructible<T>));
-    Ih_validator(const T& a) : v(a) {}
-    Ih_validator() : v(T()) {}
-    T v;
-  };
 
-  template<class T> void validate(boost::any& v, const vector<string>& xs, Ih_validator<T>*, int)
-  {
-    using namespace PO;
-    check_first_occurrence(v);
-    v = boost::any(Ih_validator<T>(boost::lexical_cast<T>(get_single_string(xs))));
-  }
+  //
+  // some custom validation code for this configuration
+  //
 
-  template<class T> ostream& operator<<(ostream& os, const Ih_validator<T>& c)
-  {
-    return os << c.v;
-  }
-
-  ostream& operator<<(ostream& os, const Ih_validator<bool>& c)
-  {
-    return os << (c.v ? "yes" : "no");
-  }
-
-  void validate(boost::any& v,
-                const std::vector<std::string>& values,
-                Ih_validator<bool>*, int)
-  {
-    using namespace PO;
-
-    check_first_occurrence(v);
-    string s(get_single_string(values, true));
-
-    for (string::size_type i = 0; i < s.size(); ++i)
-    {
-      s[i] = char(tolower(s[i]));
-    }
-
-    if (s.empty() || (s == "on") || (s == "yes") || (s == "1") || (s == "true"))
-    {
-      v = boost::any(Ih_validator<bool>(true));
-    }
-    else if ((s == "off") || (s == "no") || (s == "0") || (s == "false"))
-    {
-      v = boost::any(Ih_validator<bool>(false));
-    }
-    else
-    {
-      throw validation_error(validation_error::invalid_bool_value, s);
-    }
-  }
-
-  typedef string (*Ih_lookup_ptr)(const string&);
-
-  template<class T, Ih_lookup_ptr F> struct Ih_lookup_validator
-  {
-    BOOST_CONCEPT_ASSERT((boost::CopyConstructible<T>));
-    Ih_lookup_validator(const T& a) : v(a) {}
-    T v;
-  };
-
-  template<class T, Ih_lookup_ptr F> ostream& operator<<(ostream& os,
-                                                         const Ih_lookup_validator<T, F>& c)
-  {
-    return os << c.v;
-  }
-
-  template<class T, Ih_lookup_ptr F> void validate(boost::any& v,
-                const std::vector<std::string>& values,
-                Ih_lookup_validator<T, F>*, int)
-  {
-    using namespace PO;
-
-    validators::check_first_occurrence(v);
-    // error if more than one string supplied
-    const string& s = validators::get_single_string(values);
-
-    if ("" != F(s))
-    {
-      v = boost::any(Ih_lookup_validator<T, F>(s));
-    }
-    else
-    {
-      throw validation_error(validation_error::invalid_option_value);
-    }
-  }
-
-  typedef struct Ih_lookup_validator<string, Speech_synthesizer_config::setup_command_for_voice>
+  typedef struct Str_to_str_lookup_validator<string, Speech_synthesizer_config::setup_command_for_voice>
   Ih_voice_validator;
 
-  template<class T, int i> struct Ih_cond_validator
-  {
-    BOOST_CONCEPT_ASSERT((boost::CopyConstructible<T>));
-    BOOST_CONCEPT_ASSERT((boost::DefaultConstructible<T>));
-    Ih_cond_validator(const T& a) : v(a) {}
-    Ih_cond_validator() : v(T()) {}
-    bool cond() const;
-    T v;
-  };
-
-  template<class T, int i> ostream& operator<<(ostream& os, const Ih_cond_validator<T, i>& c)
-  {
-    return os << c.v;
-  }
-
-  template<class T, int i> void validate(boost::any& v,
-                                         const std::vector<std::string>& values,
-                                         Ih_cond_validator<T, i>*, int)
-  {
-    using namespace PO;
-
-    check_first_occurrence(v);
-    string s(get_single_string(values, true));
-
-    T a = boost::lexical_cast<T>(s);
-    Ih_cond_validator<T, i> cv(a);
-
-    if (cv.cond())
-    {
-      v = boost::any(cv);
-    }
-    else
-    {
-      throw validation_error(validation_error::invalid_option_value);
-    }
-  }
-
-  typedef struct Ih_cond_validator<int, 0> Ih_pos_int_validator;
-  template<> bool Ih_pos_int_validator::cond() const
-  {
-    return v > 0;
-  }
-  typedef struct Ih_cond_validator<double, 0> Ih_fraction_validator;
-  template<> bool Ih_fraction_validator::cond() const
-  {
-    return ((v >= 0.0) && (v <= 1.0));
-  }
+  //
+  // the main validator struct for this configuration
+  //
 
   struct Ih_validator_struct
   {
     Ih_validator_struct(const Kinetophone_vox_config& s)
-      : image_directory(s.image_directory),
+      : image_source_path(s.image_source_path),
+        image_directory(),
         vox_generation(s.vox_generation),
-        detailed_error(Ih_validator<bool>(s.detailed_error)),
-        quit_on_pregenerate_warnings(Ih_validator<bool>(s.quit_on_pregenerate_warnings)),
-        custom_installation_dir(s.custom_installation_dir),
+        detailed_error(Validator<bool>(s.detailed_error)),
+        quit_on_pregenerate_warnings(Validator<bool>(s.quit_on_pregenerate_warnings)),
+        custom_installation_dir(Directory_validator(s.custom_installation_dir)),
         speech_synthesizer_config(*s.speech_synthesizer_config) {}
     void update(Kinetophone_vox_config& return_cfg)
     {
-      return_cfg.image_directory = image_directory;
       return_cfg.detailed_error = detailed_error.v;
       vox_generation.update(return_cfg.vox_generation);
       return_cfg.detailed_error = detailed_error.v;
       return_cfg.quit_on_pregenerate_warnings = quit_on_pregenerate_warnings.v;
-      return_cfg.custom_installation_dir = custom_installation_dir;
+      return_cfg.custom_installation_dir = custom_installation_dir.v;
       speech_synthesizer_config.update(*return_cfg.speech_synthesizer_config);
+      // calculate and assign remaining members
+      if ("" == image_source_path)
+      {
+        if ("" != image_directory.v)
+        {
+          return_cfg.image_source_path = image_directory.v;
+        }
+        else
+        {
+          // left blank to indicate that the source in the session file should be used
+          return_cfg.image_source_path = "";
+        }
+      }
+      else
+      {
+        return_cfg.image_source_path = image_source_path;
+      }
     }
-    string image_directory;
+    string image_source_path;
+    Directory_validator image_directory;
     struct Vox_generation_validator_struct
     {
       Vox_generation_validator_struct(const Kinetophone_vox_config::Vox_generation_struct& s)
         : input_session_file(s.input_session_file),
           output_session_file(s.output_session_file),
-          quiet(Ih_validator<bool>(s.quiet)),
-          verbose(Ih_validator<bool>(s.verbose)),
-          skip_generate_warnings(Ih_validator<bool>(s.skip_generate_warnings)),
-          overwrite_output_session_file(Ih_validator<bool>(s.overwrite_output_session_file)),
-          output_empty_slides(Ih_validator<bool>(s.output_empty_slides)) {}
+          input_session_file_is_PDF(Validator<bool>(s.input_session_file_is_PDF)),
+          quiet(Validator<bool>(s.quiet)),
+          verbose(Validator<bool>(s.verbose)),
+          skip_generate_warnings(Validator<bool>(s.skip_generate_warnings)),
+          overwrite_output_session_file(Validator<bool>(s.overwrite_output_session_file)),
+          output_empty_slides(Validator<bool>(s.output_empty_slides)) {}
       void update(Kinetophone_vox_config::Vox_generation_struct& return_s)
       {
         return_s.input_session_file = input_session_file;
         return_s.output_session_file = output_session_file;
+        return_s.input_session_file_is_PDF = input_session_file_is_PDF.v;
         return_s.quiet = quiet.v;
         return_s.verbose = verbose.v;
         return_s.skip_generate_warnings = skip_generate_warnings.v;
@@ -233,15 +130,16 @@ namespace
       }
       string input_session_file;
       string output_session_file;
-      Ih_validator<bool> quiet;
-      Ih_validator<bool> verbose;
-      Ih_validator<bool> skip_generate_warnings;
-      Ih_validator<bool> overwrite_output_session_file;
-      Ih_validator<bool> output_empty_slides;
+      Validator<bool> input_session_file_is_PDF;
+      Validator<bool> quiet;
+      Validator<bool> verbose;
+      Validator<bool> skip_generate_warnings;
+      Validator<bool> overwrite_output_session_file;
+      Validator<bool> output_empty_slides;
     } vox_generation;
-    Ih_validator<bool> detailed_error;
-    Ih_validator<bool> quit_on_pregenerate_warnings;
-    string custom_installation_dir;
+    Validator<bool> detailed_error;
+    Validator<bool> quit_on_pregenerate_warnings;
+    Directory_validator custom_installation_dir;
     struct Speech_synthesizer_config_validator_struct
     {
       Speech_synthesizer_config_validator_struct(const Speech_synthesizer_config& s)
@@ -259,7 +157,7 @@ namespace
       struct Sound_file_config_validator_struct
       {
         Sound_file_config_validator_struct(const Sound_file_config& s)
-          : sample_rate(Ih_pos_int_validator(static_cast<int>(s.sample_rate))),
+          : sample_rate(Positive_int_validator(static_cast<int>(s.sample_rate))),
             file_name(s.file_name) {}
         void update(Sound_file_config& return_s)
         {
@@ -267,10 +165,10 @@ namespace
           return_s.file_name = file_name;
         }
         //
-        Ih_pos_int_validator sample_rate;
+        Positive_int_validator sample_rate;
         string file_name;
       } sound_file;
-      Ih_validator<bool> file_overwrite;
+      Validator<bool> file_overwrite;
     } speech_synthesizer_config;
   };
 
@@ -278,69 +176,24 @@ namespace
   // Auxiliary functions for checking input validity.
   //
 
-  // Function used to check that 'opt1' and 'opt2' are not specified
-  // at the same time.
-  void ih_check_conflicting_options(const PO::variables_map& VM,
-                                    const char* opt1,
-                                    const char* opt2,
-                                    bool condition = true,
-                                    const string& message = "")
-  {
-    precondition(opt1
-                 && opt2);
-
-    if (condition &&
-        (VM.count(opt1) && !VM[opt1].defaulted()
-         && VM.count(opt2) && !VM[opt2].defaulted()))
-    {
-      string exception_message = string("conflicting options '")
-        + opt1 + "' and '" + opt2 + "'";
-      if ("" != message)
-      {
-        exception_message += string(": ") + message;
-      }
-      exception_message += ".";
-      throw std::logic_error(exception_message);
-    }
-  }
-
-  // function used to check that 'for_what' is specified, then
-  // 'required_option' is specified too.
-  void ih_check_option_dependency(const PO::variables_map& VM,
-                                  const char* for_what,
-                                  const char* required_option,
-                                  bool condition = true)
-  {
-    precondition(for_what
-                 && required_option);
-
-    if (VM.count(for_what) && !VM[for_what].defaulted())
-    {
-      if ((VM.count(required_option) == 0)
-          || VM[required_option].defaulted()
-          || !condition)
-      {
-        throw std::logic_error(string("option '") + for_what
-                               + "' requires option '" + required_option + "'.");
-      }
-    }
-  }
-
   void ih_check_option_consistency(const Kinetophone_vox_config& cfg,
                                    const PO::variables_map& VM)
   {
     // conflicting option checks
-    ih_check_conflicting_options(VM,
-                                 "quiet",
-                                 "verbose");
+    check_conflicting_options(VM,
+                              "quiet",
+                              "verbose");
+    check_conflicting_options(VM,
+                              "image-directory",
+                              "image-source-path");
     // required option checks
     if ("" == cfg.vox_generation.input_session_file)
     {
-      throw std::logic_error(string("input session file not specified"));
+      throw logic_error(string("input session file not specified"));
     }
     if ("" == cfg.vox_generation.output_session_file)
     {
-      throw std::logic_error(string("output session file not specified"));
+      throw logic_error(string("output session file not specified"));
     }
   }
 
@@ -368,8 +221,7 @@ namespace
       ;
 
     return_desc_params.basic_desc.add_options()
-      ("voice,w", PO::value<Ih_voice_validator>(&return_v.speech_synthesizer_config.voice)
-       ->default_value(return_v.speech_synthesizer_config.voice),
+      ("voice,w", PO::value<Ih_voice_validator>(&return_v.speech_synthesizer_config.voice),
        "set voice for speech synthesizer\n"
        "{ en_br_m (British English male)\n"
        "| en_us_m (American English male)\n"
@@ -383,49 +235,54 @@ namespace
       ("output-session-file,O", PO::value<string>(&return_v.vox_generation.output_session_file)
        ->default_value(""),
        "set input session file")
-      ("image-directory,D", PO::value<string>(&return_v.image_directory)
-       ->default_value(return_v.image_directory),
+      ("image-directory,D", PO::value<Directory_validator>(&return_v.image_directory),
        "set image source directory (overrides session)")
+      ("image-source-path,P", PO::value<string>(&return_v.image_source_path)
+       ->default_value(return_v.image_source_path),
+       "set image source path (directory or PDF file, overrides session file)")
       ("sound-file,f", PO::value<string>(&return_v.speech_synthesizer_config.sound_file.file_name)
        ->default_value(return_v.speech_synthesizer_config.sound_file.file_name),
        "set destination sound file path")
-      ("overwrite,o", PO::value<Ih_validator<bool> >(&return_v.speech_synthesizer_config.file_overwrite)
-       ->implicit_value(Ih_validator<bool>(true)),
+      ("overwrite,o", PO::value<Validator<bool> >(&return_v.speech_synthesizer_config.file_overwrite)
+       ->implicit_value(Validator<bool>(true)),
        "overwrite output sound recording file")
       ("overwrite-session-file,W",
-       PO::value<Ih_validator<bool> >(&return_v.vox_generation.overwrite_output_session_file)
-       ->implicit_value(Ih_validator<bool>(true)),
+       PO::value<Validator<bool> >(&return_v.vox_generation.overwrite_output_session_file)
+       ->implicit_value(Validator<bool>(true)),
        "overwrite output session file")
       ("sample-rate,s",
-       PO::value<Ih_pos_int_validator>(&return_v.speech_synthesizer_config.sound_file.sample_rate)
+       PO::value<Positive_int_validator>(&return_v.speech_synthesizer_config.sound_file.sample_rate)
        ->default_value(return_v.speech_synthesizer_config.sound_file.sample_rate),
        "set samples per second (must be > 0)")
+      ("input-session-file-is-PDF,S",
+       PO::value<Validator<bool> >(&return_v.vox_generation.input_session_file_is_PDF)
+       ->implicit_value(Validator<bool>(true)),
+       "the file specified as the input session file is in PDF format (use its text annotations as notes)")
       ;
 
     return_desc_params.advanced_desc.add_options()
       ("skip-generate-warnings,k",
-       PO::value<Ih_validator<bool> >(&return_v.vox_generation.skip_generate_warnings)
-       ->implicit_value(Ih_validator<bool>(true)),
+       PO::value<Validator<bool> >(&return_v.vox_generation.skip_generate_warnings)
+       ->implicit_value(Validator<bool>(true)),
        "do not output vox generation warnings (cannot use with --quit-on-pregenerate-warnings)")
-      ("quiet,Q", PO::value<Ih_validator<bool> >(&return_v.vox_generation.quiet)
-       ->implicit_value(Ih_validator<bool>(true)),
-       "little output, if any, from the vox generation (cannot use with --verbose)")
-      ("verbose,V", PO::value<Ih_validator<bool> >(&return_v.vox_generation.verbose)
-       ->implicit_value(Ih_validator<bool>(true)),
+      ("quiet,Q", PO::value<Validator<bool> >(&return_v.vox_generation.quiet)
+       ->implicit_value(Validator<bool>(true)),
+       "little output, if any, from vox generation (cannot use with --verbose)")
+      ("verbose,V", PO::value<Validator<bool> >(&return_v.vox_generation.verbose)
+       ->implicit_value(Validator<bool>(true)),
        "talkative vox generation (cannot use with --quiet)")
       ("quit-on-pregenerate-warnings,p",
-       PO::value<Ih_validator<bool> >(&return_v.quit_on_pregenerate_warnings)
-       ->implicit_value(Ih_validator<bool>(true)),
+       PO::value<Validator<bool> >(&return_v.quit_on_pregenerate_warnings)
+       ->implicit_value(Validator<bool>(true)),
        "quit, do not continue, if there are pre-generate warning(s) " // (check_code_ignore)
        "(cannot use with --skip-generate-warnings)")
-      ("detailed-error,e", PO::value<Ih_validator<bool> >(&return_v.detailed_error)
-       ->implicit_value(Ih_validator<bool>(true)),
+      ("detailed-error,e", PO::value<Validator<bool> >(&return_v.detailed_error)
+       ->implicit_value(Validator<bool>(true)),
        "display detailed diagnostics on error")
-      ("config-directory,C", PO::value<string>(&return_v.custom_installation_dir)
-       ->default_value(return_v.custom_installation_dir),
+      ("config-directory,C", PO::value<Directory_validator>(&return_v.custom_installation_dir),
        "set custom program configuration directory")
-      ("output-empty-slides,E", PO::value<Ih_validator<bool> >(&return_v.vox_generation.output_empty_slides)
-       ->implicit_value(Ih_validator<bool>(true)),
+      ("output-empty-slides,E", PO::value<Validator<bool> >(&return_v.vox_generation.output_empty_slides)
+       ->implicit_value(Validator<bool>(true)),
        "output slides with no notes as segments")
       ;
   }
@@ -438,7 +295,7 @@ namespace
 Kinetophone_vox_config::Kinetophone_vox_config()
   // setup some reasonable defaults
   : help_message(""),
-    image_directory(""), // left blank to indicate that the directory in the session file should be used
+    image_source_path(""), // left blank to indicate that the directory in the session file should be used
     vox_generation(Vox_generation_struct()),
     detailed_error(false),
     quit_on_pregenerate_warnings(false),
@@ -456,6 +313,7 @@ Kinetophone_vox_config::Vox_generation_struct::Vox_generation_struct()
   // setup some reasonable defaults
   : input_session_file(""),
     output_session_file(""),
+    input_session_file_is_PDF(false),
     quiet(false),
     verbose(false),
     skip_generate_warnings(false),
@@ -520,7 +378,7 @@ int Kinetophone_vox_config::parse_program_options(int argc,
     help_stream << "Usage: " << endl;
     help_stream << "kinetophone_vox --help" << endl << endl;
     help_stream << "kinetophone_vox --list-devices" << endl << endl;
-    help_stream << "kinetophone_vox input_session_file output_session_file [sound_file ] [ options ]";
+    help_stream << "kinetophone_vox input-session-file output-session-file [sound-file ] [ options ]";
     help_stream << endl << endl;
 
     PO::options_description all_desc("Options");
